@@ -22,7 +22,7 @@
 **	arising by using whole or a part of this software.
 **********************************************************************
     convert.h	input side converter common header
-    $Id: convert.h,v 1.111 2015/04/26 15:08:38 seiji Exp seiji $
+    $Id: convert.h,v 1.118 2020/11/21 15:12:57 seiji Exp seiji $
 */
 #ifndef CONVERT_H_DEFINED
 #define CONVERT_H_DEFINED
@@ -57,6 +57,7 @@
 #define is_space(x)	((x == ' ') || (x == '\v') || (x == '\n') \
 			  || (x == '\t') || (x == '\r') || (x == '\f'))
 #define is_digit(x)	((x >= '0') && (x <= '9'))
+#define is_octal(x)	((x >= '0') && (x <= '7'))
 #define is_alpha(x)	(((x >= 'A') && (x <= 'Z')) \
 			  || ((x >= 'a') && (x <= 'z')))
 #define is_lower(x)	((x >= 'a') && (x <= 'z'))
@@ -84,6 +85,8 @@ extern int	be_in P_((skfFILE *));
 extern int	esc_process P_((skfFILE *));
 extern int	c1_process P_((skfFILE *, int));
 extern int	seq_sweep P_((skfFILE *, int));
+extern int	y_in_dec P_((int));	/* in in_converter.c	  */
+extern int	y_in_dec_m P_((int,int));	/* in in_converter.c	  */
 
 extern int	cns11643_defined_code;
 
@@ -106,6 +109,8 @@ extern void g1table2up();
 extern void g2table2up();
 extern void g3table2up();
 
+extern skf_ucode decode_surrogate P_((int,int));
+
 /* --------------------------------------------------------------- */
 /* Unicode processing						   */
 /* --------------------------------------------------------------- */
@@ -115,11 +120,14 @@ extern void g3table2up();
 #define DEC_HOOK_UTF8	1
 #define DEC_HOOK_UTF7	2
 #define DEC_HOOK_UTF32	3
+#define DEC_HOOK_UTF7I	4
 
 #define is_cod_utf16(x)	(x == DEC_HOOK_UTF16)
 #define is_cod_utf32(x)	(x == DEC_HOOK_UTF32)
 #define is_cod_utf8(x)	(x == DEC_HOOK_UTF8)
 #define is_cod_utf7(x)	(x == DEC_HOOK_UTF7)
+#define is_cod_utf7i(x)	(x == DEC_HOOK_UTF7I)
+#define is_cod_utf7m(x)	((x == DEC_HOOK_UTF7I) || (x == DEC_HOOK_UTF7))
 
 extern int	uni_in	P_((skfFILE *, int));
 extern skf_ucode u_dec_hook P_((skfFILE *, int));
@@ -219,6 +227,7 @@ extern struct iso_byte_defs priv_drcs_dblbyte_defs[];
 #define X0212_TBL_END	9025	/* must set later		   */
 
 #define X0208_KANJI_END 7896	/* After kuten 85-01 is gaiji area */
+#define X0208_KANJI_LIM 7810	/* TRUE Kanji-end in X0208 	   */
 #define X0208_CODE_END 8836	/* table end (94*94)		   */
 #define ISOMB_CODE_END 8836	/* table end (94*94)		   */
 #define ISOSB_CODE_END 128	/* table end for single byte	   */
@@ -377,13 +386,14 @@ extern int has_mime;		/* true if detect input mime	   */
 #define BV_ESC  0xff
 
 /* --------------------------------------------------------------- */
-#if 0
+#ifdef FUTURESUPPORT
 extern unsigned short uni_t_k21[];
 extern unsigned short uni_t_k22[];
 #endif
 
 extern unsigned short uni_t_x208[];
-#if 0
+
+#ifdef FUTURESUPPORT
 extern unsigned short uni_t_dia[];
 extern unsigned short uni_t_xsign[];
 extern unsigned short uni_t_xgreek[];
@@ -484,19 +494,12 @@ void shift_cond_recovery();
 /* --------------------------------------------------------------- */
 extern void	keis_conv P_((int, int));
 extern void	jef_conv P_((int, int));
-#if 0
-extern void	is_sbroken P_((int, int));
-#endif
 
 extern int	x0201conv P_((int, int));
 extern unsigned short x0201rconv P_((skf_ucode));
 
 extern int	latin2html P_((skf_ucode));
 extern int	latin2tex P_((skf_ucode));
-#if 0
-extern int	latin2htmlcnt P_((skf_ucode));
-extern int	latin2texcnt P_((skf_ucode));
-#endif
 
 extern int	is_in_encoded P_(());
 
@@ -514,9 +517,6 @@ extern short	in_keis;	/* KEIS83 mode in		   */
 #ifndef FAST_MULT
 extern int   mul94[];
 #endif
-#if 0
-extern int   is_octet;		/* '1' means octet code set	   */
-#endif
 
 extern int   low_dbyte;		/* lower side double byte(left)	   */
 extern int   up_dbyte;		/* upper side double byte(right)   */
@@ -605,6 +605,7 @@ extern char	*skf_table_lmagic;
 #define GB2312_TBL_LEN	8836
 #define GB12345_TBL_LEN	8836
 #define CP932932_PATCH_LEN 376	/* 94 * 4	*/
+#define	ISO2022MS_TABLE_LEN 1880
 
 #define BIG5_TBL_LEN	17860	/* 190 * 94 */
 #define BIG5P_TBL_LEN	23940	/* BIG5-Plus 190 * 126 */
@@ -627,12 +628,11 @@ extern char	*skf_table_lmagic;
 /* --------------------------------------------------------------- */
 /* Unic*de decomposition/composition table definitions		   */
 /* --------------------------------------------------------------- */
-/* #define UNI_NKDL_TBL_LEN 13376 :* 0x00a0-0x33ff,0xf900-0xffff   */
-#define UNI_NKDL_TBL_LEN 20480	/* 0x00a0-0x33ff,0xf900-0xffff     */
+#define UNI_NKDL_TBL_LEN 19697	/* 0x00a0-0x33ff,0xf900-0xffff     */
 				/* 0x1d100-0x1d7ff,0x2f800-0x2fa20 */
 				/* KAITHI, ARAM, ESCP etc.	   */
-
-#define UNI_NORM_TBL_LEN 20480	/* about (UNI_TAIL_OFF) * 1.1	   */
+				/*  ... and pad for one		   */
+#define UNI_NORM_TBL_LEN 19697	/* (UNI_TAIL_OFF)		   */
 
 #define UNI_LAT_OFF	160	/* 0x00a0			   */
 #define UNI_UP_OFF	13152	/* 0x33ff - 0x00a0		   */
@@ -696,8 +696,9 @@ extern void dump_table_address P_((struct iso_byte_defs *,char *));
 #define RACE_PRFX2	'q'
 
 #define is_puny_basic(x) ((punycode_uint)(x) < 0x80)
-#define is_puny_delim(x) ((x == PUNY_DELIM) || (x == '.') || (x <= 0x20))
-#define is_puny_strdelim(x) ((x == '.') || (x <= 0x20))
+#define is_puny_delim(x) ((x == PUNY_DELIM) || (x == '.') || (x <= 0x20) ||\
+		(is_prohibit(x)))
+#define is_puny_strdelim(x) ((x == '.') || (x <= 0x20) || is_prohibit(x))
 
 #define punycode_uint	skf_ucode
 #define punycode_success	0
@@ -710,10 +711,18 @@ extern void dump_table_address P_((struct iso_byte_defs *,char *));
 #define punycode_unexpeof	-7
 
 extern skf_ucode puny_adapt P_ ((long, long, int));
-extern int	is_prohibit P_ ((skf_ucode));
+extern int	 is_prohibit P_ ((skf_ucode));
 extern int	in_ace;
 #endif
 
+#define ARIB_PREDEF_MACRO_CNT	16
+#define ARIB_PREDEF_MACRO_IDX	0x60
+#ifdef ARIB_FULL_MACRO_DEF
+#define ARIB_PREDEF_MACRO_CHAR	326
+#else
+#define ARIB_PREDEF_MACRO_CHAR	71
+#endif
+
 /* --------------------------------------------------------------- */
 /* Vietnamese							   */
 /* --------------------------------------------------------------- */
@@ -743,4 +752,7 @@ extern int	in_ace;
 			  ((x == 'u') && (y == '*')) || \
 			  ((x == 'U') && (y == '*')))
 /* --------------------------------------------------------------- */
+extern int udh_parser P_((int,skfFILE *));
+extern int udh_set_lang P_((int));
+/* --------------------------------------------------------------- */
 #endif
