@@ -22,7 +22,7 @@
 **	arising by using whole or a part of this software.
 **********************************************************************
 ** error.c:	error message related routine in skf
-** $Id: error.c,v 1.149 2020/11/19 14:38:24 seiji Exp seiji $
+** $Id: error.c,v 1.152 2023/01/21 13:28:56 seiji Exp seiji $
 **	out_area_out:	error character warning generation
 **	bogus_out:	same as above
 **
@@ -509,38 +509,26 @@ int mode;
 #endif
       skf_err_msg = skf_gettext("OPTIONS: ");
       fprintf(skfstderr,skf_err_msg);
-#ifdef FAST_GETC
-      fprintf(skfstderr,"FG ");
+#ifndef FAST_GETC
+      fprintf(skfstderr,"SG ");
 #endif
-#ifdef HAVE_GETENV
-      fprintf(skfstderr,"ENV ");
-#endif
-#ifdef OLD_NEC_COMPAT
-      fprintf(skfstderr,"98 ");
+#ifndef HAVE_GETENV
+      fprintf(skfstderr,"NE ");
 #endif
 #ifdef UNIFY_ASCII_JIS
       fprintf(skfstderr,"UFY_A_J ");
 #endif
-#ifdef UTF_MIME_RECOVERY
-      fprintf(skfstderr,"MIMEREC ");
-#endif
 #if	defined(ENABLE_NLS) && defined(HAVE_LIBINTL_H)
       fprintf(skfstderr,"NLS ");
 #endif
 #ifdef USE_UBUF
       fprintf(skfstderr,"UBUF ");
 #endif
-#ifdef KEIS_DETECT
-      fprintf(skfstderr,"KD ");
+#ifndef DYNAMIC_LOADING
+      fprintf(skfstderr,"STT ");
 #endif
-#ifdef OLD_KEIS_DETECT
-      fprintf(skfstderr,"OKD ");
-#endif
-#ifdef DYNAMIC_LOADING
-      fprintf(skfstderr,"DY ");
-#endif
-#ifdef SUPPRESS_FJ_CONVENSION
-      fprintf(skfstderr,"!FJ ");
+#ifdef NOSPACECONVERT
+      fprintf(skfstderr,"SPNC ");
 #endif
 #ifdef SUPPRESS_UNRECOGNIZED_SEQ
       fprintf(skfstderr,"SUQ ");
@@ -566,9 +554,9 @@ int mode;
 #endif
 #ifdef GEN_SCRIPT_SUPRT
 #ifdef GEN_SCRIPT_ULM
-      fprintf(skfstderr,"!ULM ");
-#else
       fprintf(skfstderr,"ULM ");
+#else
+      fprintf(skfstderr,"!ULM ");
 #endif
 #endif
 #ifdef HAVE_GETUID
@@ -581,6 +569,25 @@ int mode;
 #ifndef HAVE_MKSTEMP
       fprintf(skfstderr,"OMST ");
 #endif
+#ifdef SUPPRESS_FJ_CONVENSION
+      fprintf(skfstderr,"NFJ ");
+#endif
+#if defined(SWIG_EXT) && defined(SKF_PYTHON3)
+#ifdef USE_PEP623
+    fprintf(skfstderr,"PEP623 ");
+#else
+#ifdef USE_PEP393
+    fprintf(skfstderr,"PEP393 ");
+#else
+    fprintf(skfstderr,"PY3 ");
+#endif
+#endif
+#else
+    fprintf(skfstderr,"PY2 ");
+#endif
+#if defined(SWIG_EXT) && defined(SKF_RUBY)
+    fprintf(skfstderr,"RB ");
+#endif
 #ifdef SKF_EXPERIMENTAL
       fprintf(skfstderr,"EXP ");
 #endif
@@ -588,7 +595,6 @@ int mode;
     } else;
     skf_err_msg = skf_gettext("FEATURES: ");
     fprintf(skfstderr,skf_err_msg);
-    fprintf(skfstderr,"KN ");
 #ifdef UCS2_KANA_SUPPR
     fprintf(skfstderr,"UK ");
 #endif
@@ -601,23 +607,42 @@ int mode;
 #ifdef KEIS_EXTRA_SUPPORT
     fprintf(skfstderr,"KX ");
 #endif
-#ifdef ROT_SUPPORT
+#if defined(ROT_SUPPORT)
     fprintf(skfstderr,"ROT ");
 #endif
 #ifdef ACE_SUPPORT
     fprintf(skfstderr,"ACE ");
 #endif
 #ifdef FOLD_SUPPORT
-    fprintf(skfstderr,"FD ");
 #ifdef SUPERFOLD
     fprintf(skfstderr,"SFD ");
+#else
+    fprintf(skfstderr,"FD ");
 #endif
-#if defined(SWIG_EXT) && defined(SKF_PYTHON3)
-    fprintf(skfstderr,"PY3 ");
 #endif
-#if defined(SWIG_EXT) && defined(SKF_RUBY19)
-    fprintf(skfstderr,"RBC ");
+#ifdef UTF_MIME_RECOVERY
+      fprintf(skfstderr,"MIMEREC ");
 #endif
+#ifdef OLD_NEC_COMPAT
+      fprintf(skfstderr,"98 ");
+#endif
+#ifdef ARIB_SUPPORT
+      fprintf(skfstderr,"ARIB ");
+#endif
+#ifdef FORCE_ENABLE_ARIB_MACRO_DEF
+      fprintf(skfstderr,"ARIBMAC ");
+#endif
+#ifdef KEIS_DETECT
+      fprintf(skfstderr,"KD ");
+#endif
+#ifdef OLD_KEIS_DETECT
+      fprintf(skfstderr,"OKD ");
+#endif
+#ifdef ENABLEDEPRECATED
+      fprintf(skfstderr,"ONKF ");
+#endif
+#ifdef NKF_INPLACESTAR
+      fprintf(skfstderr,"STAR ");
 #endif
     if (is_lineend_thru) fprintf(skfstderr,"LE_THRU ");
     if (is_lineend_crlf) fprintf(skfstderr,"LE_CRLF ");
