@@ -23,9 +23,10 @@
 **	arising by using whole or a part of this software.
 **********************************************************************
     in_decoder.c	input-side converter decode routines
-    $Id: in_decoder.c,v 1.71 2015/04/26 15:11:48 seiji Exp seiji $
+    $Id: in_decoder.c,v 1.79 2020/11/21 15:12:57 seiji Exp seiji $
 
-    Punycode decode part is copyrighted by The Internet society.
+    Punycode decode part is based on the work copyrighted by 
+    The Internet society.
 */
 
 #include <stdio.h>
@@ -49,7 +50,7 @@
 /* --------------------------------------------------------------- */
 /* code decorder : generic decode preprocessor			   */
 /* --------------------------------------------------------------- */
-#define	PREQDEPTH	256	/* must be 2^n			   */
+#define	PREQDEPTH	512	/* must be 2^n			   */
 #define QPAD		16
 #define	PSTQDEPTH	256	/* PSTQDEPTH > PUNY_BUFLEN	   */
 				/* also must be 2^N		   */
@@ -88,6 +89,7 @@ static int	rot_esc_hook = 0;
 
 /* --- internal work for u_dec_hook ------------------------------ */
 static int	    utf7in = 0;
+static int	    utf7hin = 0;
 static int	    res_bit = 0;
 static int	    y_res = 0;
 
@@ -124,6 +126,7 @@ static int	    y_res = 0;
 #define delete_postQ_tail() {decode_post_wptr--;}
 #define pre_Qdepth()	(decode_pre_wptr - decode_pre_rptr)
 #define post_Qdepth()   (decode_post_wptr - decode_post_rptr)
+#define pre_Q128spare	(decode_pre_wptr <= (PREQDEPTH - 129))
 
 static void post2preQueue()
 {
@@ -146,7 +149,7 @@ int pre_Qfull()
 }
 
 /*@+boolint@*/
-int pre_Qofull()
+static int pre_Qofull()
 {
     int	out_pre_rbuf = decode_pre_rptr + PREQDEPTH;
     if (decode_pre_rptr <= decode_pre_wptr) {
@@ -203,7 +206,7 @@ static void post2preQueue()
 
 #ifdef ACE_SUPPORT
 /* --------------------------------------------------------------- */
-/* internationalized IDN decoders				   */
+/* internationalized IDN decoders(RFC 3492)			   */
 /* --------------------------------------------------------------- */
 static skf_ucode puny_in_buf[PUNY_BUFLEN+16]; /* input-side buffer */
 static skf_ucode puny_out_buf[PUNY_BUFLEN+16];/* output-side buffer */
@@ -215,7 +218,7 @@ int in_ace = 0;
 #define puny_flagged(x) (x <= 'Z')
 
 /*@+boolint@*/
-int is_puny_attr_char(int x)
+static int is_puny_attr_char(int x)
 {
    if (is_digit(x) || is_alpha(x)) return(TRUE);
    return((x == '_') || (x == '.') ||
@@ -246,8 +249,8 @@ static punycode_uint decode_puny_digit(punycode_uint cp)
 /*** Main decode function ***/
 
 /* --- Punycode (RFC 3492) --- */
-/*@+boolint@*/
-int punycode_decode(input_length, input, output_length, output)
+/*@+boolint@*//*@-looploopbreak@*/
+static int punycode_decode(input_length, input, output_length, output)
   int input_length;
   const skf_ucode input[];
   int *output_length;
@@ -261,6 +264,9 @@ int punycode_decode(input_length, input, output_length, output)
   n = PUNY_INIT_N;
   out = i = 0;
   bias = PUNY_I_BIAS;
+#ifdef SKFDEBUG
+    if (is_vvv_debug) fprintf(stderr,"puny(%d): -",input_length);
+#endif
 
   /* Handle the basic code points:  Let b be the number of input code */
   /* points before the last delimiter, or 0 if there is none, then    */
@@ -348,15 +354,16 @@ int punycode_decode(input_length, input, output_length, output)
   return punycode_success;
 }
 
+#ifdef RACECODE_SUPPORT
 /* --- RACE code --- */
-int race_decode_digit(punycode_uint rc)
+static int race_decode_digit(punycode_uint rc)
 {
     return((rc < 'a') ? ((rc < '2') ? -1 : ((rc < '8') ? rc - '2'+26 : -1)) :
     	((rc <= 'z') ? (rc - 'a') : -1));
 }
 
 /*@+boolint@*/ /*@-paramuse@*/
-int racecode_decode(input_length, input, output_length, output)
+static int racecode_decode(input_length, input, output_length, output)
   punycode_uint input_length;
   const skf_ucode input[];
   punycode_uint *output_length;
@@ -372,7 +379,7 @@ int racecode_decode(input_length, input, output_length, output)
     if (is_vvv_debug) fprintf(stderr,"race(%d): -",input_length);
 #endif
     *output_length = 0;
-    for (i=0; i<input_length;) {
+    for (i=0; i<input_length ;) {
 	if ((uv = race_decode_digit(input[i++])) < 0) {
 	    break;	/* conversion end (tail or error)	   */
 	};
@@ -391,9 +398,9 @@ int racecode_decode(input_length, input, output_length, output)
 	    if ((uw = race_decode_digit(input[i++])) < 0) {
 		return punycode_bad_input; /* should not occur	   */
 	    };
-	    chu = res + uv;
-	    res = 0;
-	    b32stat = 0;
+	    chu = res + ((uv & 0x1fU) << 2) + ((uw & 0x18U) >> 3);
+	    res = ((uw & 0x07U) << 5);
+	    b32stat = 3;
 	} else if (b32stat == 3) {
 	    chu = res + uv;
 	    res = 0;
@@ -411,20 +418,20 @@ int racecode_decode(input_length, input, output_length, output)
 	    u1 = chu; wn1 = 1;
 	    if (chu == 0xd8) {
 	    	wn1 = 8; lcheck = 1;
-	    } else;
+	    };
 	} else if (wn1 == 1) {		/* Step 2		   */
 	    if (chu == 0xff) {
 		wn1 = 5; continue;
 	    };
 	    if ((u1 == 0) && (chu == 0x99)) { /* Step 3 & 4	   */
 		return(punycode_bad_input); /* expected error	   */
-	    };
+	    } else;
 	    output[*output_length] = (u1 << 8) + chu;
 	    *output_length += 1;
 	    wn1 = 1; continue; 
 	} else if (wn1 == 5) {		/* Step 5		   */
 	    if ((i == (input_length - 1)) && (b32stat == 0)) {
-	    	return(punycode_bad_input);
+		return(punycode_bad_input); /* expected error	   */
 	    } else if (chu == 0x99) {
 		output[*output_length] = (u1 << 8) + 0xff;
 		*output_length += 1;
@@ -440,13 +447,13 @@ int racecode_decode(input_length, input, output_length, output)
 	    *output_length += 1;
 	    lcheck = 1; continue;
 	} else {	/* LCHECK start  case			   */
-	    u1 = chu;
-	    lcheck = 1; continue; /* skip lcheck validate, for	   */
-			  /* such check in skf is worthless	   */
+	    lcheck = 1; u1 = chu;
+	    continue;
 	}; 
     };
     return (punycode_success);
 }
+#endif	/* RACECODE_SUPPORT */
 #endif
 
 /* --------------------------------------------------------------- */
@@ -454,13 +461,16 @@ int racecode_decode(input_length, input, output_length, output)
 /* --------------------------------------------------------------- */
 static int mime_res_tbl[5] = { 4,0,0,0,2 };
 
-/*@-globstate@*/
-void low2convtbl()
+static void low2convtbl()
+/*@globals low_table,low_ltable,low_table_limit,low_dbyte,low_table_mod,
+	low_kana,skf_output_lang,skf_input_lang;@*/
+/*@modifies low_table,low_ltable,low_table_limit,low_dbyte,
+	low_kana,skf_output_lang,skf_input_lang;@*/
 {
     low_table = low_table_mod->unitbl;
     low_ltable = low_table_mod->uniltbl;
     low_table_limit = low_table_mod->table_len;
-    low_dbyte = low_table_mod->char_width - 1;
+    low_dbyte = is_multibyte(low_table_mod->char_width);
     low_kana = cod_kana_mask(low_table_mod->is_kana);
     if (low_table_mod->lang != L_NU) {
 	  skf_input_lang = skf_get_wlang(low_table_mod->lang);
@@ -468,25 +478,26 @@ void low2convtbl()
 	      skf_output_lang = skf_input_lang;
 	      show_lang_tag();
 	  }; };
-    if (((low_table == NULL) && (low_dbyte > 0) && (low_dbyte < 3))
-      || ((low_ltable == NULL) && (low_dbyte >= 3))) {
+    if ((is_tblshort(low_table_mod->char_width) && (low_table == NULL))||
+    	(is_tbllong(low_table_mod->char_width) && (low_ltable == NULL))) {
 	skferr(SKF_LOW2ERRDUMP,(long)0,(long)0);
-	skf_exit(EXIT_FAILURE);
     };
 }
 
-/*@-globstate@*/
-void up2convtbl()
+static void up2convtbl()
+/*@globals up_table,up_ltable,up_table_limit,up_dbyte,up_table_mod,
+	up_kana;@*/
+/*@modifies up_table,up_ltable,up_table_limit,up_dbyte,
+	up_kana;@*/
 {
     up_table = up_table_mod->unitbl;
     up_ltable = up_table_mod->uniltbl;
     up_table_limit = up_table_mod->table_len;
-    up_dbyte = up_table_mod->char_width - 1;
+    up_dbyte = up_table_mod->char_width;
     up_kana = cod_kana_mask(up_table_mod->is_kana);
-    if (((up_table == NULL) && (up_dbyte > 0) && (up_dbyte < 3))
-      || ((up_ltable == NULL) && (up_dbyte >= 3))) {
+    if (((up_table == NULL) && (is_tblshort(up_dbyte)))
+      || ((up_ltable == NULL) && (is_tbllong(up_dbyte)))) {
 	skferr(SKF_UP2ERRDUMP,(long)0,(long)0);
-	skf_exit(EXIT_FAILURE);
     };
 }
 
@@ -494,93 +505,121 @@ void up2convtbl()
 void g0table2low()
 {
     if (g0_table_mod != NULL) {	/* it is null when input undefined */
-      if (((g0_table_mod->char_width > 2) &&
+      if ((is_tbllong(g0_table_mod->char_width) &&
 		(g0_table_mod->uniltbl != NULL)) ||
 	  (g0_table_mod->unitbl != NULL)) {
 	  low_table_mod = g0_table_mod;
 	  low2convtbl();
       };
+      if (is_charset_macro(low_table_mod) == 1) set_macl;
+      else res_macl;
     };
 }
 
-/*@-globstate@*/
 void g1table2low()
 {
     if (g1_table_mod != NULL) {	/* it is null when input undefined */
-      if (((g1_table_mod->char_width > 2) &&
+      if ((is_tbllong(g1_table_mod->char_width) &&
 		(g1_table_mod->uniltbl != NULL)) ||
 	  (g1_table_mod->unitbl != NULL)) {
 	  low_table_mod = g1_table_mod;
 	  low2convtbl();
       };
+      if (is_charset_macro(low_table_mod) == 1) set_macl;
+      else res_macl;
     };
 }
 
-/*@-globstate@*/
 void g2table2low()
 {
     if (g2_table_mod != NULL) {	/* it is null when input undefined */
-      if (((g2_table_mod->char_width > 2) &&
+      if ((is_tbllong(g2_table_mod->char_width) &&
 		(g2_table_mod->uniltbl != NULL)) ||
 	  (g2_table_mod->unitbl != NULL)) {
 	  low_table_mod = g2_table_mod;
 	  low2convtbl();
       };
+      if (is_charset_macro(low_table_mod) == 1) set_macl;
+      else res_macl;
     };
 }
 
-/*@-globstate@*/
 void g3table2low()
 {
     if (g3_table_mod != NULL) {	/* it is null when input undefined */
-      if (((g3_table_mod->char_width > 2) &&
+      if ((is_tbllong(g3_table_mod->char_width) &&
 		(g3_table_mod->uniltbl != NULL)) ||
 	  (g3_table_mod->unitbl != NULL)) {
 	  low_table_mod = g3_table_mod;
 	  low2convtbl();
       };
+      if (is_charset_macro(low_table_mod) == 1) set_macl;
+      else res_macl;
     };
 }
 
-/*@-globstate@*/
+void g0table2up()
+{
+    if (g0_table_mod != NULL) {
+      if ((is_tbllong(g0_table_mod->char_width) &&
+		(g0_table_mod->uniltbl != NULL)) ||
+	  (g0_table_mod->unitbl != NULL)) {
+	  up_table_mod = g0_table_mod;
+	  up2convtbl();
+      } else;
+      if (is_charset_macro(up_table_mod) == 1) set_macr;
+      else res_macr;
+    } else;
+}
+
 void g1table2up()
 {
     if (g1_table_mod != NULL) {
-      if (((g1_table_mod->char_width > 2) &&
+      if ((is_tbllong(g1_table_mod->char_width) &&
 		(g1_table_mod->uniltbl != NULL)) ||
 	  (g1_table_mod->unitbl != NULL)) {
 	  up_table_mod = g1_table_mod;
 	  up2convtbl();
       } else;
+      if (is_charset_macro(up_table_mod) == 1) set_macr;
+      else res_macr;
     } else;
 }
 
-/*@-globstate@*/
 void g2table2up()
 {
     if (g2_table_mod != NULL) {
-      if (((g2_table_mod->char_width > 2) &&
+      if ((is_tbllong(g2_table_mod->char_width) &&
 		(g2_table_mod->uniltbl != NULL)) ||
 	  (g2_table_mod->unitbl != NULL)) {
 	  up_table_mod = g2_table_mod;
 	  up2convtbl();
       };
+      if (is_charset_macro(up_table_mod) == 1) set_macr;
+      else res_macr;
     };
 }
 
-/*@-globstate@*/
 void g3table2up()
 {
     if (g3_table_mod != NULL) {
-      if (((g3_table_mod->char_width > 2) &&
+      if ((is_tbllong(g3_table_mod->char_width) &&
 		(g3_table_mod->uniltbl != NULL)) ||
 	  (g3_table_mod->unitbl != NULL)) {
 	  up_table_mod = g3_table_mod;
 	  up2convtbl();
       };
+      if (is_charset_macro(up_table_mod) == 1) set_macr;
+      else res_macr;
     };
 }
 
+skf_ucode decode_surrogate(c1,c2)
+int	c1,c2;
+{
+    return((((c1 & U_SRMSK) + 0x40) << 10) + (c2 & U_SRMSK));
+}
+
 #if	defined(ROT_SUPPORT) && defined(NEW_ROT_CODE)
 /* --------------------------------------------------------------- */
 /* for rot13 ---------------- */
@@ -645,7 +684,8 @@ char	*sh,*sb;
     if ((*sh == 'x') && (*(sh + 1) == '-')) sh += 2;
     if ((*sb == 'x') && (*(sb + 1) == '-')) sb += 2;
 
-    while ((*sh != '\0') && (*sb != '\0') && (++i < MIME_CSET_LEN)) {
+    while ((*sh != '\0') && (*sb != '\0') && (++i < MIME_CSET_LEN)
+		&& (*sh != ' ')) {
 	ch = SKFtolower(*sh); /* with canonicalization		   */
 	if (!is_cname_char(ch)) {
 	    break;
@@ -663,6 +703,7 @@ char	*sh,*sb;
 	sb++; sh++;
     };
     if ((*sh == '\0') && (*sb == '\0')) return(1);
+    else if ((*sh == ' ') && (*sb == '\0')) return(1);
     else if (*sb == '\0') return(0);
     else return(-1);
 }
@@ -782,11 +823,9 @@ int	*sy;
     };
     mime_cname[MIME_CSET_LEN - 1] = '\0'; /* add terminator	   */
     /* call search routines */
-    if ((sss = skf_search_cname(mime_cname)) < 0) { /* not found   */
-	if ((sss = skf_option_parser(mime_cname,
-				codeset_option_code)) < 0) {
+    if ((sss = skf_search_cname_w_alias(mime_cname)) < 0) {
+    				/* not found			   */
 	    in_codeset = DEFAULT_I; return(-2);
-	};
     };
     in_codeset = sss;
     return(0);
@@ -799,7 +838,7 @@ int decode_hook(f,flg)
 skfFILE *f;
 int	flg;
 {
-    int c1, c2 = 0, c3, ch = EOF, c4;
+    int c1, c2 = 0, c3, ch = sEOF, c4;
     int inp;
     int qmark = 0;
     int	pres = 0;		/* parse result			   */
@@ -843,7 +882,7 @@ int	flg;
 	|| ((is_hex_uri || (is_hex_encoded && !is_hex_cap)) &&
 		(c1 == '%'))
 	|| (is_rfc2231_encoded && in_rfc2231 && (c1 == '%')))
-	&& ((low_dbyte == 0) || (uri_esc_hook == 0))) {
+	&& ((!low_dbyte) || (uri_esc_hook == 0))) {
 			/* *************************************** */
 			/* hex-cap-uri expr. decode		   */
 			/* *************************************** */
@@ -855,7 +894,6 @@ int	flg;
 #endif
 	  if (pre_Qofull()) {
 		skferr(SKF_DECERRDUMP,decode_pre_wptr,decode_pre_rptr);
-		skf_exit(EXIT_FAILURE);
 	  };
 	  if (!is_hex_char(c3)) {
 	      pre_Qenque(c3); continue;   /* broken		   */
@@ -981,15 +1019,9 @@ int	flg;
 	} else {
 	    mime_res_bit = 0; return(c1); 
 	}; 
-#if 0
-      } else if (((is_mimeb_encoded) || (is_mimeb_strict) 
-		 || (is_mimeq_encoded)) 
-		 && (!is_in_ucs_ufam || in_mime)) {
-#else
       } else if (((is_mimeb_encoded) || (is_mimeb_strict) 
 		 || (is_mimeq_encoded)) 
 		 && (!is_in_ucs_ufamnomime || in_mime)) {
-#endif
 			/* *************************************** */
 			/* MIME decoder 			   */
 			/* *************************************** */
@@ -1151,7 +1183,7 @@ int	flg;
 #ifdef SKFDEBUG
 	    if (is_vv_debug) fprintf(stderr," peeking:");
 #endif
-	    while ((c3 = unhook_getc(f,flg)) != EOF) {
+	    while ((c3 = unhook_getc(f,flg)) != sEOF) {
 #ifdef SKFDEBUG
 		if (is_vv_debug) fprintf(stderr,"%02x,",c3);
 #endif
@@ -1313,7 +1345,7 @@ int	flg;
 	    post_Qclear(); pre_Qclear(); in_mime = 0; 
 #ifdef NEW_SOFTWRAP_CODE
 	    mime_mark = 0;
-	    while ((c3 = unhook_getc(f,flg)) != EOF) {
+	    while ((c3 = unhook_getc(f,flg)) != sEOF) {
 	    	if (pre_Qfull()) {
 		    pre_Qenque(c3);
 		    break;	/* real out. output as it is.	   */
@@ -1492,7 +1524,7 @@ int	flg;
 		unGETC(c1,f); return(sOCD);
 	    };
 	    post_Qclear(); post_Qenque(c1);
-	    if ((c2 = hook_q_getc(f,flg)) == EOF) continue;
+	    if ((c2 = hook_q_getc(f,flg)) == sEOF) continue;
 	    if (((SKFtolower(c1) != PUNY_PRFX1) 
 			|| (SKFtolower(c2) != PUNY_PRFX2))
 		&& ((SKFtolower(c1) != RACE_PRFX1)
@@ -1502,7 +1534,7 @@ int	flg;
 	    if ((c3 = hook_q_getc(f,flg)) != '-') continue;
 	    /* ACE label is confirmed now. go to buffering	   */
 	    w = 0;
-	    while ((c3 = hook_getc(f,flg)) != EOF) {
+	    while ((c3 = hook_getc(f,flg)) != sEOF) {
 		if (is_label_delim(c3)) break;
 #ifdef SKFDEBUG
 		if (is_vvv_debug) fprintf(stderr,"ace_in: %x ",c3);
@@ -1516,21 +1548,16 @@ int	flg;
 	    if ((SKFtolower(c1) == PUNY_PRFX1) &&  /* PUNY	   */
 	    	((pres = punycode_decode(w,puny_in_buf,&(outchar),
 		puny_out_buf)) >= 0)) {/* throw to Qbuffer */
+#ifdef RACECODE_SUPPORT
 	    } else if ((SKFtolower(c1) == RACE_PRFX1) &&  /* RACE  */
 	    	((pres = racecode_decode(w,puny_in_buf,&(outchar),
 		puny_out_buf)) >= 0)) {/* throw to Qbuffer */
+#endif
 	    } else {	
 		in_undefined((0x4000-pres),SKF_DECODERR);
 		continue;
 	    };
 
-#ifdef SKFDEBUG
-	    if (is_vvv_debug) {
-		fprintf(stderr,"ace_out: %d,%lx ",
-			outchar,(unsigned long)puny_out_buf);
-		(void)fflush(stderr);
-	    };
-#endif
 	    post_Qclear();
 	    for (k=0; k < outchar; k++) {
 		rcod = puny_out_buf[k];
@@ -1539,7 +1566,7 @@ int	flg;
 	    };
 	    pre_Qenque(c3);  /* re-supply delimiter	  */
 	    codeset_push_for_mime();
-	    in_codeset = codeset_utf16be;
+	    in_codeset = codeset_utf16le;
 	    set_dummy_detect;
 	    set_in_endian;		/* set big endian	   */
 	    in_ace = TRUE;
@@ -1566,7 +1593,6 @@ int	flg;
 #endif
 	  if (pre_Qofull()) {
 		skferr(SKF_DECERRDUMP2,decode_pre_wptr,decode_pre_rptr);
-		skf_exit(EXIT_FAILURE);
 	  };
 	  if (!is_hex_char(c3)) {
 	      if (is_space(c3) || is_lineend(c3)) {
@@ -1592,7 +1618,7 @@ int	flg;
 		encode_cap,in_mime,pre_Qdepth(),post_Qdepth());
 #endif
 	if (((low_dbyte && (c1 < 0x80) && (c1 > A_SP)) ||
-	    (up_dbyte && (c1 >= 0x80))) && !rot_esc_hook) {
+	    (is_multibyte(up_dbyte) && (c1 >= 0x80))) && !rot_esc_hook) {
 	    post_Qenque(c1); 
 	    if ((c2 = hook_q_getc(f,flg)) == sEOF) continue;
 	    post_Qunenque(); /* c2 */
@@ -1694,20 +1720,25 @@ int cod;
     int	d1,d2,d3;
     skf_ucode ch = 0;
     int	cm;
+    int mod;
 
     while (TRUE) {
 	if ((c1 = vGETC(f)) < 0) return(c1);
 #ifdef SKFDEBUG
-	if (is_vv_debug) fprintf(stderr,"\nudh_raw: %02x",c1);
+	if (is_vv_debug) fprintf(stderr,"\nudh_raw(%x): %02x",cod,c1);
 #endif
-	if (is_cod_utf7(cod)) {	/* Unic*de UTF-7		   */
+	if (is_cod_utf7m(cod)) {/* Unic*de UTF-7/UTF7-IMAP	   */
 #ifdef SKFDEBUG
 	  if (is_vv_debug) fprintf(stderr," U7");
 #endif
 	    c1 &= 0x7fU;	/* reset bit8			   */
-	    if (utf7in) {		/* in base64		   */
+	    mod = is_cod_utf7i(cod);
+	    if ((utf7hin) && (c1 == '-')) { /* UTF7-IMAP detect	   */
+	        ch = 0x26; utf7hin = FALSE;
+		utf7in = FALSE;
+	    } else if (utf7in) {	/* in base64		   */
 		if ((c1 == '-') ||
-		    (y_in_dec(c1) < 0)) {  /* escape from base64   */
+		    (y_in_dec_m(c1,mod) < 0)) { /* escape from base64 */
 		    if (c3 != 0) in_undefined(c3,SKF_IUNDEF);
 		    if (c2 != 0) in_undefined(c2,SKF_IUNDEF);
 		    c2 = 0; c3 = 0; res_bit = 0; /* it should be 0  */
@@ -1742,11 +1773,21 @@ int cod;
 		if (is_vvv_debug) fprintf(stderr,
 			" y_in: %04x(%01d,%04x)",ch,res_bit,y_res);
 #endif
+		if ((is_cod_utf7i(cod)) && (ch >= 0x20) && (ch < A_DEL)) {
+		    in_undefined(ch,SKF_IBROKEN);
+		    ch = 0x20;
+		} else;
 		c2 = 0; c3 = 0; c1 = 0; 
-	    } else if (c1 == '+') {  /* UTF7 escape character	   */
+	    } else if ((c1 == '+') && (mod == FALSE)) {
+	    			/* UTF7 escape character	   */
 		utf7in = TRUE; 
 		c2 = 0; c3 = 0; res_bit = 0; y_res = 0;
 		continue;
+	    } else if ((c1 == '&') && (mod == TRUE)) {
+	    			/* UTF7-IMAP escape character	   */
+		utf7in = TRUE; utf7hin = TRUE;
+		c2 = 0; c3 = 0; res_bit = 0; y_res = 0;
+		continue;
 	    } else {		/* within normal ascii's	   */
 		ch = c1;
 	    };
@@ -1755,7 +1796,10 @@ int cod;
 	  if (is_vv_debug) fprintf(stderr," U8");
 #endif
 	    if (c1 <= A_DEL) {		/* 0x00 - 0x7f		   */
-		return(c1);
+	    	if ((c1 == 0) && (in_codeset == codeset_jmutf8)) {
+		    in_undefined(c1,SKF_IBROKEN);
+		    continue;
+		} else return(c1);
 	    } else if ((cm = (c1 & ZF2MSK)) < ZF23VAL) {
 		in_undefined(c1,SKF_IBROKEN);
 		continue;
@@ -1766,6 +1810,14 @@ int cod;
 		    if (is_vvv_debug) fprintf(stderr," z_in_raw: %02x",c2);
 #endif
 		    ch = ((c1 & Z23MSK) << 6) | (c2 & Z1MSK);
+		} else if ((c1 == ZF23VAL) &&
+			   (in_codeset == codeset_jmutf8)) {
+		    GETC_add(c2,c1,f);
+		    ch = ((c1 & Z23MSK) << 6) | (c2 & Z1MSK);
+		    if (ch != 0) {
+			in_undefined(c1,SKF_IBROKEN);
+			continue;
+		    } else;
 		} else {
 		    in_undefined(c1,SKF_IBROKEN);
 		    continue;
@@ -1786,7 +1838,10 @@ int cod;
 		    continue;
 		};
 		ch |= (c3 & Z1MSK);
-		if (((ch >= 0xd800) && (ch < 0xe000)) && !enable_cesu) {
+		if (((ch >= 0xd800) && (ch < 0xe000)) && 
+		    (!enable_cesu &&
+		    (in_codeset != codeset_jmutf8) &&
+		    (in_codeset != codeset_cesu8) )) {
 		    in_undefined(ch,SKF_IBROKEN);
 		    continue;
 		};
@@ -1808,9 +1863,14 @@ int cod;
 		if ((c1 == 0xf0) && (c2 < 0x90) && !non_strict_utf8) {
 		    in_undefined((c1 << 8) + c2,SKF_IBROKEN);
 		    continue;
-		};
+		} else;
 		ch = ((c1 & Z4MSK) << 18) | ((c2 & Z43MSK) << 12)
 		    | ((c3 & Z1MSK) << 6) | (c4 & Z1MSK);
+		if ((in_codeset == codeset_cesu8) ||
+		    (in_codeset == codeset_jmutf8)) {
+		    in_undefined(ch,SKF_IBROKEN);
+		    continue;
+		} else;
 	    } else ch = 0;
 	} else if (is_cod_utf32(cod)) {	/* utf-32 (a.k.a. UCS4)	   */
 #ifdef SKFDEBUG
@@ -1865,7 +1925,7 @@ int cod;
 		c3 = c1; c1 = c2; c2 = c3;
 	    };
 	    if ((c1 >= 0xd8) && (c1 <=0xdf)){ /* surrogate pair	   */
-		ch = (((c1 << 8) + c2) & U_SRMSK) + 0x40; 
+		ch = ((c1 << 8) + c2);
 		if ((c4 = vGETC(f)) == sEOF) {
 		    in_undefined(ch, SKF_NOSURRG);
 		};
@@ -1875,8 +1935,8 @@ int cod;
 		if (in_endian) {  /* swap if big endian		   */
 		    c1 = c4; c4 = c3; c3 = c1;
 		};
-		c2 = ((c3 << 8) + c4) & U_SRMSK ;
-		ch = (ch << 10) + c2;
+		c2 = (c3 << 8) + c4;
+		ch = decode_surrogate(ch,c2);
 	    } else 			/* normal case		   */
 		ch = ((c1 & 0xffU) << 8) | (c2 & 0xffU);
 	};
@@ -1922,3 +1982,102 @@ void clear_after_mime()
 /* clear all mime status */
     mime_res_bit = 0; mime_y_res = 0; sp_mime = 0; 
 }
+/* --------------------------------------------------------------- */
+/* paraphrase_arib_macro:					   */
+/* --------------------------------------------------------------- */
+/* TODO: should check macro invoke doesn't contain some chars.	   */
+/* --------------------------------------------------------------- */
+#define is_macro_low(x) ((x > A_SP) && (x < A_DEL))
+#define is_macro_up(x)  ((x > A_KSP) && (x < 0xff))
+
+int paraphrase_arib_macro(ch)
+int ch;
+{
+    int i,sy,sz;
+    int *macstr;
+    int g0mac = FALSE;
+    int g1mac = FALSE;
+    int g2mac = FALSE;
+    int g3mac = FALSE;
+    int glmac = FALSE;
+    int gumac = FALSE;
+    int inssg2 = FALSE;
+    int inssg3 = FALSE;
+
+#ifdef SKFDEBUG
+    if (is_vv_debug) {
+    	fprintf(stderr,"ARIB_MACRO: ch-%c(",ch);
+    } else;
+#endif
+    if ((ch <= A_SP) || (ch > A_DEL)) return(ch);
+    if (arib_macro_tbl == NULL) return(ch);
+    if ((macstr = arib_macro_tbl[ch - 0x21]) == NULL) {
+	/* no macro defined for this character			   */
+	return(ch);
+    } else {
+	/* has macro definition and <128 length			   */
+        if (is_charset_macro(g0_table_mod)) g0mac = TRUE;
+        if (is_charset_macro(g1_table_mod)) g1mac = TRUE;
+        if (is_charset_macro(g2_table_mod)) g2mac = TRUE;
+        if (is_charset_macro(g3_table_mod)) g3mac = TRUE;
+        if (is_charset_macro(low_table_mod)) glmac = TRUE;
+        if (is_charset_macro(up_table_mod)) gumac = TRUE;
+
+    	for (i=0;(i<128) && (macstr[i] != '\0');i++) {
+	    sy = macstr[i];
+#ifdef SKFDEBUG
+	    if (is_vv_debug) {
+		fprintf(stderr,"%02x,",sy);
+	    } else;
+#endif
+	    if (inssg2) {
+	    	/* actually not correct in iso-2022 aspect */
+	        if ((g2mac) && (sy > A_SP) && (sy < A_DEL)) 
+			return(0);	/* terminate		   */
+		inssg2 = FALSE;
+		inssg3 = FALSE;
+	    } else if (inssg3) {
+	    	/* actually not correct in iso-2022 aspect */
+	        if ((g3mac) && (sy > A_SP) && (sy < A_DEL)) 
+			break;		/* terminate		   */
+		inssg3 = FALSE;
+	    } else if (is_macro_low(sy)) {
+	    	if (sy == A_SO) {
+		    glmac = g1mac; continue;
+		} else if (sy == A_SI) {
+		    glmac = g0mac; continue;
+		} else if (sy == A_ESC) {
+		    sz = macstr[++i];
+		    if (sz == 0x6e) { /* LS2 */
+		    	glmac = g2mac; continue;
+		    } else if (sz == 0x6f) { /* LS3 */
+		    	glmac = g3mac; continue;
+		    } else if (sz == 0x7e) { /* LS1R */
+		    	gumac = g1mac; continue;
+		    } else if (sz == 0x7d) { /* LS2R */
+		    	gumac = g2mac; continue;
+		    } else if (sz == 0x7c) { /* LS3R */
+		    	gumac = g3mac; continue;
+		    } else;
+		} else if (glmac) {
+		    ;	/* pass them */
+		} else; /* pass others */
+	    } else if (is_macro_up(sy)) {
+	        if (sy == A_SS2) {
+		    inssg2 = TRUE; continue;
+		} else if (sy == A_SS3) {
+		    inssg3 = TRUE; continue;
+		} else if (gumac) {
+		    ;	/* pass them */
+		} else;
+	    } else; /* not within GL nor GR - pass as it is */
+	    enque(macstr[i]);
+	};
+    };
+#ifdef SKFDEBUG
+    if (is_vv_debug) {
+    	fprintf(stderr,")\n");
+    } else;
+#endif
+    return(0);
+}
