@@ -22,7 +22,7 @@
 **	arising by using whole or a part of this software.
 **********************************************************************
 ** oconv.c:	unic*de post-handler. called from in_converter.
-** $Id: oconv.c,v 1.144 2015/04/26 15:08:38 seiji Exp seiji $
+** $Id: oconv.c,v 1.154 2020/11/19 14:38:24 seiji Exp seiji $
 **
 **********************************************************************
 ** Notice for limitation of these converter programming.
@@ -57,30 +57,30 @@ static  int	lang_tag_index = 0;
 /* --------------------------------------------------------------- */
 /* code output definitions					   */
 /* --------------------------------------------------------------- */
-unsigned int	g0_char = 'J';	/* g0 iso-2022-char		   */
-unsigned int	g0_mid = '(';	/* g0 iso-2022-char mid call	   */
-unsigned int	g0_midl = 0;	/* g0 iso-2022-char 4th call	   */
+	 int	g0_char = 'J';	/* g0 iso-2022-char		   */
+	 int	g0_mid = '(';	/* g0 iso-2022-char mid call	   */
+	 int	g0_midl = 0;	/* g0 iso-2022-char 4th call	   */
 unsigned long	g0_typ = 0;	/* g0 iso-2022-char type	   */
-unsigned int	ag0_char = 0;	/* ag0 iso-2022-char		   */
-unsigned int	ag0_mid = '$';	/* ag0 iso-2022-char mid call	   */
-unsigned int	ag0_midl = '(';	/* ag0 iso-2022-char 4th call	   */
+	 int	ag0_char = 0;	/* ag0 iso-2022-char		   */
+	 int	ag0_mid = '$';	/* ag0 iso-2022-char mid call	   */
+	 int	ag0_midl = '(';	/* ag0 iso-2022-char 4th call	   */
 unsigned long	ag0_typ = COD_MB;	/* ag0 iso-2022-char type   */
-unsigned int	g1_char = 0;	/* g1 iso-2022-char		   */
-unsigned int	g1_mid = 0x2a;	/* g1 iso-2022-char mid call	   */
-unsigned int	g1_midl = 0;	/* g1 iso-2022-char 4th call	   */
+	 int	g1_char = 0;	/* g1 iso-2022-char		   */
+	 int	g1_mid = 0x2a;	/* g1 iso-2022-char mid call	   */
+	 int	g1_midl = 0;	/* g1 iso-2022-char 4th call	   */
 unsigned long	g1_typ = COD_SET96;	/* g1 iso-2022-char type   */
-unsigned int	g2_char = 0;	/* g2 iso-2022-char		   */
-unsigned int	g2_mid = '(';	/* g2 iso-2022-char mid call	   */
-unsigned int	g2_midl = 0;	/* g2 iso-2022-char 4th call	   */
+	 int	g2_char = 0;	/* g2 iso-2022-char		   */
+	 int	g2_mid = '(';	/* g2 iso-2022-char mid call	   */
+	 int	g2_midl = 0;	/* g2 iso-2022-char 4th call	   */
 unsigned long	g2_typ = 0;	/* g2 iso-2022-char type	   */
-unsigned int	g3_char = 0;	/* g3 iso-2022-char		   */
-unsigned int	g3_mid = '$';	/* g3 iso-2022-char mid call	   */
-unsigned int	g3_midl = '(';	/* g3 iso-2022-char 4th call	   */
+	 int	g3_char = 0;	/* g3 iso-2022-char		   */
+	 int	g3_mid = '$';	/* g3 iso-2022-char mid call	   */
+	 int	g3_midl = '(';	/* g3 iso-2022-char 4th call	   */
 unsigned long	g3_typ = COD_MB | COD_MB_4;
 				/* g3 iso-2022-char type	   */
-unsigned int	g4_char = 'P';	/* g4 iso-2022-char		   */
-unsigned int	g4_mid = '$';	/* g4 iso-2022-char mid call	   */
-unsigned int	g4_midl = '(';	/* g4 iso-2022-char 4th call	   */
+	 int	g4_char = 'P';	/* g4 iso-2022-char		   */
+	 int	g4_mid = '$';	/* g4 iso-2022-char mid call	   */
+	 int	g4_midl = '(';	/* g4 iso-2022-char 4th call	   */
 unsigned long	g4_typ = COD_MB | COD_MB_4;
 				/* g4 iso-2022-char type	   */
 /* --------------------------------------------------------------- */
@@ -106,10 +106,6 @@ int		fold_count = 0;	   /* linewise character count	   */
 skf_ucode	sgbuf = 0;		/* oconv single buffer	   */
 int		sgbuf_buf = 0;
 
-skf_ucode	oobuf[OOBUFSIZE];  /* MIME/fold buffer		   */
-int		oobufip = 0;	/* write pointer		   */
-int		oobufop = 0;	/* read pointer			   */
-
 #ifdef FOLD_SUPPORT
 static int	is_intag = FALSE; /* within HTML tags		   */
 #endif
@@ -119,7 +115,7 @@ static int	is_intag = FALSE; /* within HTML tags		   */
 static int		decompose_recursion_depth = 0;
 static skf_ucode	decompose_buf[32];
 static int		decompose_bufp;
-static void		decompose_code_dec P_((unsigned short));
+static void		decompose_code_dec P_((int));
 
 #ifdef UNI_ENCOMPOSE
 static skf_ucode	encompose_buf[ENCOMPOSE_DEPTH];
@@ -238,38 +234,33 @@ const char kinsoku_map0[256] = {
 skf_ucode	prev_ch = 0;
 #endif
 
+#ifdef TSCII_SUPPORT
 /* --------------------------------------------------------------- */
-/* oobuf enque/deque						   */
+/* tscii 0x0b95 - 0x0bb9 masks for TSCII Consonants		   */
 /* --------------------------------------------------------------- */
-skf_ucode pokeoobuf()
-{
-    int po;
-    po = oobufop;
-    oobufop = (oobufop == (OOBUFSIZE - 1)) ? 0 : (oobufop + 1);
-    return(oobuf[po]);
-}
+unsigned short tscii_bcmask[38] = {
+     		    1, 0, 0,  0, 2, 3, 0,19, 0, 4, 5,
+     0, 0, 0, 6, 7, 0, 0, 0,  8,18, 9, 0, 0, 0,10,11,
+    12,17,13,16,15,14, 0,20, 21,22, 0
+};
+#endif
 
-void pushoobuf(ch)
-skf_ucode ch;
-{
-    oobuf[oobufip] = ch;
-    oobufip = (oobufip == (OOBUFSIZE - 1)) ? 0 : (oobufip + 1);
-}
-
-#define oobufempty()	(oobufip == oobufop)
-#define oobufhas1()	((oobufip == (oobufop + 1)) \
-		|| ((oobufip == 0) && (oobufop == (OOBUFSIZE - 1))))
-
-#define oobuffull()	((oobufop == (oobufip + 1)) \
-		|| ((oobufop == 0) && (oobufip == (OOBUFSIZE - 1))))
 /* --------------------------------------------------------------- */
 /* code output initializer					   */
 /* --------------------------------------------------------------- */
+/* Do not initialize output file specific feature here.		   */
+/* --------------------------------------------------------------- */
 /*@-globstate@*/
 int	oconv_init()
 {
 /* undefine character fix */
-    if (out_codeset < 0) return(0);
+    if ((out_codeset < 0) || (out_codeset >= codeset_end)) {
+    	out_codeset = DEFAULT_O;
+	skf_charset_parser(out_codeset);
+    } else if (is_cod_nooc(i_codeset[out_codeset].oconv_type)) {
+    	out_codeset = DEFAULT_O;
+	skf_charset_parser(out_codeset);
+    } else;
     if ((pref_subst_char > 0) && (test_out_char(pref_subst_char) != 0)) {
 	ucode_undef = pref_subst_char;
     } else if (!is_ucs_ufam(conv_cap)) { /* NOT ucs, brgt, trans   */
@@ -287,12 +278,10 @@ int	oconv_init()
 	if (is_euc_51932(conv_cap)) {
 	    set_kana_call;
 	};
-    } else {
-	if (use_uni_repl) {
-	    ucode_undef = 0xfffd;
-	} else {
+    } else {	/* ucs, brgt, trans */
+	if (ucode_undef <= 0) {
 	    ucode_undef = 0x3013;
-	};
+	} else;
     };
 
     if (is_o_encode) mime_limit_set();
@@ -300,7 +289,8 @@ int	oconv_init()
     if (is_o_encode) {
 /* encoding and MIME */
     /* do not set encoding if GL part is not iso-2022 like code */
-    	if (is_ucs_utf7(conv_cap) || is_brgt(conv_cap)
+	  
+    	if (is_ucs_utf7m(conv_cap) || is_brgt(conv_cap)
 	  || is_ucs_uri(conv_cap)
 	  || is_ucs_transp(conv_cap)
 	  || (in_codeset == codeset_transp)
@@ -310,10 +300,13 @@ int	oconv_init()
 	  /* these output encodings do not support MIMEs	*/
 	  out_undefined(sFLSH,SKF_ENC_ERR);
 	  o_encode = 0;
-	} else if ((out_codeset == codeset_binary)
-	  || (out_codeset == codeset_nyukan_utf8)
+	} else if ((out_codeset == codeset_nyukan_utf8)
 	  || (out_codeset == codeset_nyukan_utf16)
 	  || (out_codeset == codeset_locale)
+	  || (out_codeset == codeset_aribb24s)
+	  || (out_codeset == codeset_aribb24)
+	  || (out_codeset == codeset_aribb24sa)
+	  || (out_codeset == codeset_aribb24a)
 	  || (out_codeset == codeset_puny)
 	  || (out_codeset == codeset_transp)
 	  || (out_codeset == codeset_gsm0338)
@@ -337,6 +330,9 @@ int	oconv_init()
 	  || (out_codeset == codeset_kr)) {
 	  out_undefined(sFLSH,SKF_ENC_ERR);
 	  o_encode = 0;
+	} else if (is_cod_noenc(i_codeset[out_codeset].oconv_type)) {
+	  out_undefined(sFLSH,SKF_ENC_ERR);
+	  o_encode = 0;
 	} else;
 	if (is_o_encode_hex(o_encode) 
 		|| (is_o_encode_oct(o_encode))) 
@@ -348,6 +344,7 @@ int	oconv_init()
 /* encoding and inquiry */
 	if (input_inquiry) {
 	  o_encode = 0;
+	  o_encode_stat = FALSE;
 	};
     } else;
 
@@ -380,7 +377,22 @@ int	oconv_init()
     /* Unicord normalization */
     unicode_normalize_setup();
 #endif
+    /* -------------------------- */
+    /* VISCII/VSQR Escape overlay */
+    /* -------------------------- */
+ /* TODO */
 
+    /* -------------------------- */
+    /* TSCII modifiers overlay	  */
+    /* -------------------------- */
+#ifdef TSCII_SUPPORT
+    if (is_tscii(conv_cap)) {
+	if (load_external_table(&ovlay_byte_defs[tscii_i_index]) < 0) {
+	    in_tablefault(SKF_PRESETFAIL,ovlay_byte_defs[tscii_i_index].desc);
+	    return(-1);
+	} else;
+    };
+#endif
     return (0);
 }
 
@@ -408,6 +420,7 @@ void oconv(ch)			/* unic*de output parsing	   */
 skf_ucode ch;
 {
     skf_ucode	c1;
+    skf_ucode	rch,vch;
     skf_ucode	c2;
     int		dd;
     int		de;
@@ -417,6 +430,9 @@ skf_ucode ch;
     unsigned short uukp;
 #endif
     unsigned short cnvpk;
+#ifdef TSCII_SUPPORT
+    int		ts;
+#endif
 
 #ifdef FOLD_SUPPORT
     int		charProp = 0;	/* character property buffer	   */
@@ -440,6 +456,8 @@ skf_ucode ch;
 #endif
     };
 #endif
+    rch = ch; vch = 0;
+
     if (sgbuf_buf >= 1) {
 	if (ch < 0) {
     	    if (ch == sOCD) return;
@@ -449,7 +467,6 @@ skf_ucode ch;
 	    } else {
 	    	return;
 	    };
-#ifndef USE_1995L
 	    if ((ch >= 0xff01) && (ch <= 0xff9f)) {
 		if ((ch >= 0xff61) && (ch <= 0xff9f) && 
 		    (is_nkf_no_hk ||
@@ -458,7 +475,7 @@ skf_ucode ch;
 		    return;
 		} else if ((ch >= 0xff01) && (ch <= 0xff5e) && 
 		    (is_nkf_no_hk || is_o_prefer_ascii)) {
-		    ch -= (0xff00 + 0x20);	/* convert to ascii	*/
+		    ch -= (0xff00 + 0x20);
 		    /* pass to later process */
 		} else;
 	    } else;
@@ -468,24 +485,30 @@ skf_ucode ch;
     /* ----------------------------------------------------------- */
     /* ----------------------------------------------------------- */
 	    if (!dsbl_ucod_encomp || enbl_ucod_cencomp) {
-		if ((ch >= 0x3099) && (ch <= 0x309c) && (sgbuf_buf > 0) &&
-		    (sgbuf >= 0x3046) && (sgbuf <= 0x30f2)) {
+		if ((ch >= 0x3099) && (ch <= 0x309c) && (sgbuf_buf > 0)
+		    && (sgbuf >= 0x3046) && (sgbuf <= 0x30f2)) {
 	    /* --------------------------------------------------- */
 	    /* unicode kana normalized unification		   */
 	    /* --------------------------------------------------- */
 		    c1 = ((sgbuf >= 0x30a0) ? 
 			    (sgbuf - 0x30a0) : (sgbuf - 0x3040));
 		    if (((ch == U_KDAK) || (ch == U_LDAK)) &&
-			    (u_dakuten[c1] > 0)) {
+			    (u_dakuten[c1] > 0) &&
+		    	    (!dsbl_uni_kana_concat)) {
+			vch = ch;
 			if (c1 == 0x06) sgbuf += 0x4e;
 			else if (sgbuf >= 0x30ef) sgbuf += 8;
 			else sgbuf += 1;
 			ch = sgbuf; sgbuf_buf = 0;
 		    } else if (((ch == U_KHDK) || (ch == U_LHDK)) &&
-			    (u_dakuten[c1] == 3)) {
+			    (u_dakuten[c1] == 3) &&
+		    	    (!dsbl_uni_kana_concat)) {
+			vch = ch;
 			ch = sgbuf + 2; sgbuf_buf = 0;
-		    } else if (((ch == U_KHDK) || (ch == U_LHDK)) &&
-			    (u_dakuten[c1] == 4)) {
+		    } else if ((ch == U_KHDK) && (u_dakuten[c1] == 4) &&
+		    	    (!dsbl_uni_kana_concat ||
+			    	enbl_uni_hain_concat)) {
+			vch = ch;
 			switch (sgbuf) {
 			    case 0x304b: ch = 0xd801; sgbuf_buf = 0; break;
 			    case 0x30ab: ch = 0xd808; sgbuf_buf = 0; break;
@@ -501,44 +524,49 @@ skf_ucode ch;
 			    case 0x30c4: ch = 0xd80e; sgbuf_buf = 0; break;
 			    case 0x30c8: ch = 0xd80f; sgbuf_buf = 0; break;
 			    default:
-				c1 = ch; ch = sgbuf;
-				sgbuf = c1;
+				ch = sgbuf;
+				sgbuf = rch;
 			};
 		    } else {
-			c1 = ch; ch = sgbuf;
-			sgbuf = c1;
+			ch = sgbuf;
+			sgbuf = rch;
 		    };
 		} else if ((sgbuf >= 0x00e6) && (sgbuf < 0x300) &&
 			   ((ch == 0x300) || (ch == 0x301))) {
 		    if ((sgbuf == 0x00e6) && (ch == 0x300)) {
-		    	ch = 0xd814; sgbuf_buf = 0;
+			vch = ch; ch = 0xd814; sgbuf_buf = 0;
 		    } else if (sgbuf == 0x0254) {
-		    	ch = 0xd818 + ((ch == 0x301) ? 1 : 0);
+			vch = ch; ch = 0xd818 + ((ch == 0x301) ? 1 : 0);
 			sgbuf_buf = 0;
 		    } else if (sgbuf == 0x028c) {
-		    	ch = 0xd81a + ((ch == 0x301) ? 1 : 0);
+			vch = ch; ch = 0xd81a + ((ch == 0x301) ? 1 : 0);
 			sgbuf_buf = 0;
 		    } else if (sgbuf == 0x0259) {
-		    	ch = 0xd81c + ((ch == 0x301) ? 1 : 0);
+			vch = ch; ch = 0xd81c + ((ch == 0x301) ? 1 : 0);
 			sgbuf_buf = 0;
 		    } else if (sgbuf == 0x025a) {
-		    	ch = 0xd81e + ((ch == 0x301) ? 1 : 0);
+			vch = ch; ch = 0xd81e + ((ch == 0x301) ? 1 : 0);
 			sgbuf_buf = 0;
 		    } else {
-			c1 = ch; ch = sgbuf;
-			sgbuf = c1;
+			ch = sgbuf;
+			sgbuf = rch;
 		    };
 		} else if ((sgbuf == 0x2e9) && (ch == 0x2e5)) {
-		    ch = 0xd820; sgbuf_buf = 0;
+		    vch = ch;
+		    ch = 0xd820; sgbuf_buf = 0; 
 		} else if ((sgbuf == 0x2e5) && (ch == 0x2e9)) {
+		    vch = ch;
 		    ch = 0xd821; sgbuf_buf = 0;
 		} else if ((sgbuf == 0x31f7) && 
 			   ((ch == U_KHDK) || (ch == U_LHDK))) {
+		    vch = ch;
 		    ch = 0xd807; sgbuf_buf = 0;
 		} else if ((sgbuf >= 0xff61) && (sgbuf <= 0xff9f) &&
+		    (!dsbl_uni_hkna_concat) &&
 		    (is_nkf_no_hk ||
 		     (!hk_enbl && !(use_compat && is_ucs_ufam(conv_cap))))) {
 		    dd = sgbuf; sgbuf = sOCD; sgbuf_buf = 0;
+		    vch = ch;
 		    if ((ch == U_CDAK) || (ch == U_CHDK)) {
 			de = ch - 0xff40;
 		    } else if ((ch >= U_KDAK) && (ch <= U_LHDK)) {
@@ -548,10 +576,76 @@ skf_ucode ch;
 			sgbuf = ch; sgbuf_buf = 1;
 		    };
 		    return;
+		} else if ((sgbuf >= 0xd800) && (sgbuf < 0xd812)) {
+		    vch = 0x309a; ch = sgbuf; sgbuf = rch;
+#ifdef TSCII_SUPPORT
+		} else if (is_tscii(conv_cap)) {
+		    if ((ch >= 0xbc6) && (ch <= 0xbc8)) { /* rev   */
+			post_oconv(ch);
+			post_oconv(sgbuf);
+			sgbuf_buf = 0; sgbuf = 0;
+			return;
+		    } else if ((sgbuf == 0x0b95) && (ch == 0xbcd)) {
+		    	sgbuf = 0xdb42;
+#ifdef SKFDEBUG
+			if (is_vv_debug) fprintf(stderr,"-> 0xdb42 ");
+#endif
+			return;
+		    } else if ((sgbuf >= 0x0b95) && (sgbuf < 0x0bb9) &&
+			  ((de = tscii_bcmask[sgbuf - 0xb95]) > 0) &&
+			  ((ch == 0xbcd) || (ch == 0xbc1) || (ch == 0xbc2))){
+			    /* Mey, Ukara & Uukaara */
+			ts = (ch == 0xbcd) ? 3 : ((ch == 0xbc1) ? 4 : 5);
+			if ((ch == 0xbcd) || (de <= 18)) {
+			    sgbuf = ts * 22 + de - 1 + 0xdb00;
+			    return;
+			} else;
+		    } else if ((sgbuf >= 0x0b95) && (sgbuf < 0x0bb9) &&
+			  ((de = tscii_bcmask[sgbuf - 0xb95]) > 0) &&
+			  ((ch == 0xbca) || (ch == 0xbcb) || (ch == 0xbcc))){
+			    	/* 2-part vowels   */
+			ch = (ch - 0xbca) * 22 + de - 1 + 0xdb00;
+			sgbuf_buf = 0; sgbuf = 0;
+		    } else if ((sgbuf == 0xb9f) && (ch == 0xbbf)) { 
+			ch = 0xdb84;		/* di		   */
+			sgbuf_buf = 0; sgbuf = 0;
+		    } else if ((sgbuf == 0xb9f) && (ch == 0xbc0)) {
+			ch = 0xdb85;		/* dii		   */
+			sgbuf_buf = 0; sgbuf = 0;
+		    } else if ((sgbuf == 0xdb42) && (ch == 0xbb7)) {
+				    /* 0x0b95 + 0x0bcd + 0x0bb7	   */
+			sgbuf = 0xdb86; 	/* KSH or KSHA	   */
+#ifdef SKFDEBUG
+			if (is_vv_debug) fprintf(stderr,"-> 0xdb86 ");
+#endif
+			return;
+		    } else if ((sgbuf == 0xdb86) && (ch == 0xbcd)) {
+			ch = 0xdb87;	/* KSH			   */
+			sgbuf_buf = 0; sgbuf = 0;
+		    } else if ((sgbuf == 0xbb6) && (ch == 0xbcd)) {
+			sgbuf = 0xdb8a;	 /* tscii sri part	   */
+#ifdef SKFDEBUG
+			if (is_vv_debug) fprintf(stderr,"-> 0xdb8a ");
+#endif
+			return;
+		    } else if ((sgbuf == 0xdb8a) && (ch == 0xbb0)) {
+				    /* 0x0bb6 + 0x0bcd + 0x0bb0	   */
+			sgbuf = 0xdb89;	 /* tscii sri part	   */
+#ifdef SKFDEBUG
+			if (is_vv_debug) fprintf(stderr,"-> 0xdb89 ");
+#endif
+			return;
+		    } else if ((sgbuf == 0xdb89) && (ch == 0xbc0)) {
+			ch = 0xdb88;	/* TRUE SRI		   */
+			sgbuf_buf = 0; sgbuf = 0;
+		    } else {
+			ch = sgbuf;
+			sgbuf = rch;
+		    };
+#endif
 		} else {
-		    c1 = ch;
 		    ch = sgbuf;
-		    sgbuf = c1;
+		    sgbuf = rch;
 		};
 		if (is_kanaconv_x0201 && 
 		    (((ch >= 0x3041) && (ch <= 0x309c)) ||
@@ -569,7 +663,6 @@ skf_ucode ch;
 		    } else;
 		};
 	    } else; 
-#endif
 	};
     } else {
 	if (ch < 0) {
@@ -828,51 +921,6 @@ skf_ucode ch;
 	};
     } else if (ch <= 0x4dff) {	/* 0x80 - 0x4dff		   */
 	if (ch >= 0x3000) {
-#ifdef USE_1995L
-    /* ----------------------------------------------------------- */
-    /* --- Note: This is a point where all normalize packing	   */
-    /* 		process should be inserted.			   */
-    /* ----------------------------------------------------------- */
-	    if (!dsbl_ucod_encomp) {
-		if ((ch >= 0x3099) && (ch <= 0x309c) &&
-		    (sgbuf >= 0x3046) && (sgbuf <= 0x30f2)) {
-	    /* --------------------------------------------------- */
-	    /* unicode kana normalized unification		   */
-	    /* --------------------------------------------------- */
-		    c1 = ((sgbuf >= 0x30a0) ? 
-			    (sgbuf - 0x30a0) : (sgbuf - 0x3040));
-		    if (((ch == U_KDAK) || (ch == U_LDAK)) &&
-			    (u_dakuten[c1] > 0)) {
-			if (c1 == 0x06) sgbuf += 0x4e;
-			else if (sgbuf >= 0x30ef) sgbuf += 8;
-			else sgbuf += 1;
-		    } else if (((ch == U_KHDK) || (ch == U_LHDK)) &&
-			    (u_dakuten[c1] == 3)) {
-			sgbuf += 2; 
-		    } else {
-			c1 = ch; ch = sgbuf;
-			sgbuf = c1;
-		    };
-		} else if ((sgbuf >= 0xff61) && (sgbuf <= 0xff9f) &&
-		    !hk_enbl && !(use_compat && is_ucs_ufam(conv_cap))) {
-		    dd = sgbuf; sgbuf = sOCD; sgbuf_buf = 0;
-		    if ((ch == U_CDAK) || (ch == U_CHDK)) {
-			de = ch - 0xff40;
-		    } else if ((ch >= U_KDAK) && (ch <= U_LHDK)) {
-			de = ch - U_KDAK + 0x20;
-		    } else de = 0xb0;	/* dummy		   */
-		    if ((x = x0201conv((dd - 0xff40),de)) != 0) {
-			sgbuf = ch; sgbuf_buf = 1;
-		    };
-		    return;
-		} else {
-		    c1 = ch;
-		    ch = sgbuf;
-		    sgbuf = c1;
-		};
-	    } else; 
-	  };
-#endif
 /* nkf-compatible katakana-hiragana-convert */
 	  if ((ch >= 0x3041) && (ch <= 0x3096)) {
 		if (is_nkf_c_katakana) ch += 0x60L;
@@ -900,7 +948,7 @@ skf_ucode ch;
 #ifdef UNI_DECOMPOSE
 	    if (enbl_decomp && (ch < 0x3400)) {
 	    	if ((uukp = nkdc_lowptr[ch - UNI_LAT_OFF]) != 0) {
-		    decompose_code(uukp);
+		    decompose_code((int)uukp);
 		    return;
 		};
 	    };
@@ -918,12 +966,12 @@ skf_ucode ch;
 #endif
 	    fold_count += 2;
 	    out_undefined(ch,SKF_IOUTUNI);
-	} else {
+	} else {	/* 0xa0 - 0x2fff */
 #ifdef UNI_DECOMPOSE
 	    if (enbl_decomp) {
 	    	if (((uukp = nkdc_lowptr[ch - UNI_LAT_OFF]) != 0) &&
 	    (!decomp_apple || ((ch > 0x2adc) || (ch < 0x2000)))) {
-		    decompose_code(uukp);
+		    decompose_code((int)uukp);
 		    return;
 		};
 	    } else;
@@ -945,7 +993,9 @@ skf_ucode ch;
 	      } else fold_count ++;
 	    };
 #endif
-	    o_latin_conv(ch);
+	    if ((ch >= 0x2f00) && (ch < 0x2fe0) && cod_meiryo_bug) {
+		o_cjk_conv(uni_k_radical[ch - 0x2f00]);
+	    } else o_latin_conv(ch);
 	    fold_count++;
 #ifdef	FOLD_SUPPORT
 	    if (is_sentence_clip && 
@@ -1001,7 +1051,8 @@ skf_ucode ch;
 		} else;
 	    } else;
 	} else;
-	o_private_conv(ch);
+	if ((vch >= 0xd800) && (vch < 0xe000)) o_private_conv(ch,0);
+	else o_private_conv(ch,vch);
 	fold_count += 2;
     } else if (ch < 0x10000) {	/* compatibility plane		  */
 #ifdef	FOLD_SUPPORT
@@ -1031,7 +1082,7 @@ skf_ucode ch;
 	if (enbl_decomp) {
 	    if ((uukp = nkdc_lowptr[ch - 0xf900 + UNI_UP_OFF]) != 0) {
 	    	if (!decomp_apple || ((ch >= 0xfb1d) && (ch <= 0xfb4e))) {
-		    decompose_code(uukp);
+		    decompose_code((int)uukp);
 		    return;
 		} else;
 	    } else;
@@ -1060,36 +1111,40 @@ skf_ucode ch;
     } else if ((ch >= 0x0e0100) && (ch <= 0x0e01ff)) {
 			/* Variation selector supprement	  */
 		;	/* just discard				  */
+    } else if (ch > UNI_HIGH_LIMIT) {
+    			/* not within UTF-16, 32, 8	*/
+	out_undefined(ch,SKF_UNSURG);
+	return;
     } else {
 #ifdef UNI_DECOMPOSE
 	if (enbl_decomp && !decomp_apple) {
 	    if ((ch >= 0x1d100) && (ch < 0x1d800)) {
 		if ((uukp = nkdc_lowptr[ch - 0x1d100 + UNI_COM_OFF]) != 0) {
-		    decompose_code(uukp);
+		    decompose_code((int)uukp);
 		    return;
 		};
 	    } else if ((ch >= 0x11090) && (ch < 0x11600)) {
 	    	/* KAITHI and other codes */
 		if ((uukp = nkdc_lowptr[ch - 0x11090 + UNI_KAITHI_OFF]) != 0) {
-		    decompose_code(uukp);
+		    decompose_code((int)uukp);
 		    return;
 		};
 	    } else if ((ch >= 0x1ee00) && (ch < 0x1ef00)) {
 	    	/* arabic mathmatical addendums */
 		if ((uukp = nkdc_lowptr[ch - 0x1ee00 + UNI_ARAM_OFF]) != 0) {
-		    decompose_code(uukp);
+		    decompose_code((int)uukp);
 		    return;
 		};
 	    } else if ((ch >= 0x1f100) && (ch < 0x1f400)) {
 	    	/* Enclosed CJK addendums and emoticons */
 		if ((uukp = nkdc_lowptr[ch - 0x1f100 + UNI_ECSP_OFF]) != 0) {
-		    decompose_code(uukp);
+		    decompose_code((int)uukp);
 		    return;
 		};
 	    } else if ((ch >= 0x2f800) && (ch < 0x2fa20)) {
 			    /* CJK cpt.sup  */
 		if ((uukp = nkdc_lowptr[ch - 0x2f800 + UNI_CJKC_OFF]) != 0) { 
-		    decompose_code(uukp);
+		    decompose_code((int)uukp);
 		    return;
 		};
 	    };
@@ -1154,7 +1209,7 @@ skf_ucode ch;
     } else if (ch <= 0xd7ff) {	/* hangul area			  */
 	o_ozone_conv(ch);
     } else if (ch <= 0xf8ff) {	/* private use			  */
-	o_private_conv(ch);
+	o_private_conv(ch,0);
     } else if (ch < 0x10000) {	/* compatibility plane		  */
 	o_compat_conv(ch);
     } else if ((ch >= 0x0e0100) && (ch <= 0x0e01ff)) {
@@ -1192,14 +1247,15 @@ skf_ucode x;
     };
 }
 
-void o_private_conv(x)	
+void o_private_conv(x,rx)	
 skf_ucode x;
+skf_ucode rx;
 {
     if (is_jiscat(conv_cap)) {
 	if (is_jis(conv_cap)) JIS_private_oconv(x); 
 	else EUC_private_oconv(x);
     } else if (is_ucs_ufam(conv_cap)) {
-    	UNI_private_oconv(x);
+    	UNI_private_oconv(x,rx);
     } else if (out_ocat) {
 	if (is_msfam(conv_cap)) SJIS_private_oconv(x);
 	else if (out_bg(conv_cap)) BG_private_oconv(x);
@@ -1495,12 +1551,13 @@ skf_ucode c3;
 	    ((int) (d2 + ((d1 & 1) ? ((d2 < 0x60) ? 0x1f : 0x20) : 0x7e))));
 	};
 #endif
-    } else if ((is_ms_932(conv_cap) || is_ms_943(conv_cap)) &&
-    		(c3 <= 0xa878)) {
-    	d1 = ((c3 & 0x7f00U) >> 8) - 0x21 + 0x80 + 1;
-	d2 = (c3 & 0x7f);
-	SKFputc((int)(((d1 - 1) >> 1) + 0xb0));
-	SKFputc((int)(d2 + ((d1 & 1) ? ((d2 < 0x60) ? 0x1f : 0x20) : 0x7e)));
+    } else if (is_ms_932(conv_cap) || is_ms_943(conv_cap)) {
+    	if (c3 <= 0xa878) {
+	    d1 = ((c3 & 0x7f00) >> 8) - 0x21 + 0x80 + 1;
+	    d2 = (c3 & 0x7f);
+	    SKFputc((int)(((d1 - 1) >> 1) + 0xb0));
+	    SKFputc((int) (d2 + ((d1 & 1) ? ((d2 < 0x60) ? 0x1f : 0x20) : 0x7e)));
+	} else out_undefined(c3,SKF_OUNDEF);
     } else 	/* should not be come here			   */
 	out_undefined(c3,SKF_OUNDEF);
 }
@@ -1578,6 +1635,8 @@ skf_ucode c3;
 	SKFputc(A_SS2);
 	SKFputc((((c3) & 0x7f00) >> 8) | 0x80); 
 	SKFputc(((c3) & 0xff) | 0x80);
+    } else if (is_euc_51932(conv_cap) || is_euc_20932(conv_cap)) {
+	out_undefined(c3,SKF_OUNDEF);
     } else {
 	if (is_euc7(conv_cap)) {
 	    if (is_kanji_shift) { 
@@ -1632,9 +1691,13 @@ skf_ucode c3;
 #endif
 
     d2 = ((c3) & 0x007f); 
+#ifdef TSCII_SUPPORT
+    if (is_big5fam(conv_cap) || is_tscii(conv_cap)) {
+#else
     if (is_big5fam(conv_cap)) {
+#endif
 	SKFputc(c3);
-    } else if (is_hz(conv_cap)) {
+    } else if (is_hz(conv_cap) || is_hz8(conv_cap)) {
 	if (is_hz_shift) {
 	    SKFputc(HZ_ANN); SKFputc(HZ_END);
 	};
@@ -1720,6 +1783,9 @@ skf_ucode c3;
 	    };
 	};
     } else if (is_hz(conv_cap)) {
+#ifdef SKFDEBUG
+	    if (is_vv_debug) fprintf(stderr,"HZ ");
+#endif
 	if (!is_hz_shift) {
 	    SKFputc(HZ_ANN); SKFputc(HZ_ENT);
 	};
@@ -1738,6 +1804,15 @@ skf_ucode c3;
 	if (c3 < 0x8000) c1 |= 0x80;
 	SKFputc((int) (c2 + 0x80));
 	SKFputc((int) (c1));
+    } else if (is_hz8(conv_cap)) {
+#ifdef SKFDEBUG
+	    if (is_vv_debug) fprintf(stderr,"HZ8 ");
+#endif
+	if (!is_hz_shift) {
+	    SKFputc(HZ_ANN); SKFputc(HZ_ENT);
+	};
+	set_hz_shift;
+	SKFputc(c2 | 0x80); SKFputc(c1 | 0x80);
     } else {
 	SKFputc((int) ('.'));
     }; 
@@ -1882,29 +1957,54 @@ skf_ucode c3;
 void SKFSTROUT(x)
 const char *x;
 {
-	if (is_jis(conv_cap)) SKFJISSTROUT(x); 
-	else if (is_msfam(conv_cap)) SKFSJISSTROUT(x);
-	else if (is_euc(conv_cap)) SKFEUCSTROUT(x);
-	else if (out_bg(conv_cap)) SKFBGSTROUT(x);
-	else if (is_ucs_ufam(conv_cap)) SKFUNISTROUT(x);
-	else if (is_brgt(conv_cap)) SKFBRGTSTROUT(x);
-	else if (is_keis(conv_cap)) SKFKEISSTROUT(x); 
+	if (is_jiscat(conv_cap)) { 
+	    if (is_jis(conv_cap)) { SKFJISSTROUT(x); 
+	    } else SKFEUCSTROUT(x);
+	} else if (is_ucs_ufam(conv_cap)) {
+	    SKFUNISTROUT(x); 
+	} else if (out_ocat) {
+	    if (is_msfam(conv_cap)) { SKFSJISSTROUT(x);
+	    } else if (out_bg(conv_cap)) { SKFBGSTROUT(x);
+	    } else if (is_keis(conv_cap))  {SKFKEISSTROUT(x);
+	    } else SKFBRGTSTROUT(x);
+	} else {
+	    SKFEUCSTROUT(x);
+	}; 
 }
 
+/* --------------------------------------------------------------- */
+void SKFNSTROUT(x,len)
+char *x;
+int	len;
+{
+    int i;
+    for (i=0; (i<len) && (i<FIXSTRLEN_LIMIT); i++) {
+    	if (*(x+i) != '\0') break;
+    };
+    if ((i == len) || (i == FIXSTRLEN_LIMIT)) {
+    	skf_openerr(x,7);
+    } else;
+    SKFSTROUT((const char *)x);
+}
 /* --------------------------------------------------------------- */
 void SKF1FLSH()
 {
 #ifdef SKFDEBUG
 	if (is_vvv_debug) fprintf(stderr," FCEFLSH");
 #endif
-	if (is_jis(conv_cap)) { SKFJIS1FLSH(); 
-	} else if (is_msfam(conv_cap)) { SKFSJIS1FLSH();
-	} else if (is_euc(conv_cap)) { SKFEUC1FLSH();
-	} else if (out_bg(conv_cap)) { SKFBG1FLSH();
-	} else if (is_ucs_ufam(conv_cap)) { SKFUNI1FLSH();
-	} else if (is_brgt(conv_cap)) { SKFBRGT1FLSH();
-	} else if (is_keis(conv_cap)) { SKFKEIS1FLSH(); 
-	} else;
+	if (is_jiscat(conv_cap)) { 
+	    if (is_jis(conv_cap)) { SKFJIS1FLSH(); 
+	    } else SKFEUC1FLSH();
+	} else if (is_ucs_ufam(conv_cap)) {
+	    SKFUNI1FLSH(); 
+	} else if (out_ocat) {
+	    if (is_msfam(conv_cap)) { SKFSJIS1FLSH();
+	    } else if (out_bg(conv_cap)) { SKFBG1FLSH();
+	    } else if (is_keis(conv_cap)) { SKFKEIS1FLSH();
+	    } else SKFBRGT1FLSH();
+	} else { 
+	    SKFEUC1FLSH();
+	}; 
 }
 
 #if defined(ROT_SUPPORT) && defined(NEW_ROT_CODE)
@@ -1983,7 +2083,14 @@ void SKFCRLF()
     if (is_keis(conv_cap)) {
     	ox_ascii_conv(A_NEL);
     } else if (is_lineend_thru) {
-	if (use_first_detect_cr && use_detect_cr) {
+	if (input_inquiry) {
+#if !defined(DEFAULT_EOL_LF)
+	    ox_ascii_conv(A_CR);
+#endif
+#if defined(DEFAULT_EOL_LF) || defined(DEFAULT_EOL_CRLF)
+	    ox_ascii_conv(A_LF);
+#endif
+	} else if (use_first_detect_cr && use_detect_cr) {
 	    ox_ascii_conv(A_CR);
 	    if (use_detect_lf) ox_ascii_conv(A_LF);
 	} else {
@@ -2013,7 +2120,7 @@ void output_language_tag_trigger()	/* set read output tags	   */
 /* use sgbuf_buf, sgbuf implicitly.				   */ 
 /* --------------------------------------------------------------- */
 void decompose_code(uu)
-unsigned short uu;
+int	uu;
 {
     int i;
     int str0;
@@ -2043,7 +2150,7 @@ unsigned short uu;
 }
 
 void decompose_code_dec(uu)
-unsigned short uu;
+int uu;
 {
     int i;
     skf_ucode uukk;
@@ -2084,7 +2191,7 @@ unsigned short uu;
 	    if ((decompose_recursion_depth++) > 32) {
 	    	/* Something in table is REALLY WRONG	*/
 		skferr(SKF_DECOMPERR,(long)uu,0);
-	    } else decompose_code_dec(uujj);
+	    } else decompose_code_dec((int)uujj);
 	    decompose_recursion_depth--;
 	} else {
 	    decompose_buf[decompose_bufp++] = uukk;
@@ -2212,7 +2319,7 @@ int	oank;
 /* --------------------------------------------------------------- */
 #ifdef SKFDEBUG
 void debugcharout(c)
-char c;
+int c;
 {
     if ((c>= 0x20) && (c <= 0x7e)) {
     	fprintf(stderr,"(%c)",(int)c);
@@ -2227,7 +2334,7 @@ void GBKR_finish_procedure()
 	if (is_zw_shift) {
 	    SKFputc(ZW_LF); res_zw_shift;
 	} else;
-    } else if (is_hz(conv_cap)) {
+    } else if (is_hz(conv_cap) || is_hz8(conv_cap)) {
 	if (is_hz_shift) {
 	    SKFputc(HZ_ANN); SKFputc(HZ_END);
 	};
