@@ -22,7 +22,7 @@
 **	arising by using whole or a part of this software.
 **********************************************************************
     oconv.h:	Code conversion table related definitions
-    $Id: oconv.h,v 1.112 2015/04/26 15:08:38 seiji Exp seiji $
+    $Id: oconv.h,v 1.118 2020/11/19 14:38:24 seiji Exp seiji $
 **/
 #ifndef	OCONV_H_DEFINED
 #define OCONV_H_DEFINED
@@ -36,7 +36,6 @@
 #define X213NEW		'Q'	/* JIS X-0213(2004) plane 1	   */
 				/* Note: plane 2 is not updated	   */
 
-extern char	ascii_mask[];		/* ascii treatment mask	   */
 extern int	fold_count;
 
 extern int	o_encode_stat;
@@ -47,6 +46,7 @@ extern skf_ucode pref_subst_char;
 				/* cf. rfc2045			   */
 
 #define OOBUFSIZE	256
+#define MIME_CNAME_SIZE	32
 
 /* --------------------------------------------------------------- */
 /* output side table for kana region and kanji region		   */
@@ -221,7 +221,10 @@ extern unsigned short *uni_o_hist;
 extern unsigned short *uni_o_note;
 extern unsigned short *uni_o_upmisc;
 
+extern skf_ucode      *uniuni_o_prv;
+
 /* table feature */
+#ifdef FUTURESUPPORT
 extern unsigned long	ascii_mod;
 extern unsigned long	latin_mod;
 extern unsigned long	symbol_mod;
@@ -235,6 +238,7 @@ extern unsigned long	uni_cjkb_mod;
 extern unsigned long	uni_cjkc_mod;
 extern unsigned long	uni_upkana_mod;
 extern unsigned long	uni_prv_mod;
+#endif
 
 /* *****************************************************************
 ** Converters
@@ -245,10 +249,12 @@ extern unsigned long	uni_prv_mod;
 ** conversion tables */
 /* --------------------------------------------------------------- */
 /* --- table anchor ---------------------------------------------- */
+#ifdef FUTURESUPPORT 
 extern unsigned short *uni2latin;
 extern unsigned short *uni2math;
 extern unsigned short *uni2cjk;
 extern unsigned short *uni2compat;
+#endif
 /* --------------------------------------------------------------- */
 /* --- Actual table from unic*de to jis, euc and Sjis ------------ */
 extern unsigned short uni_f_4e[];	/* from unic*de to x208	   */
@@ -272,8 +278,8 @@ extern unsigned short uni_f_compat[];
 extern const unsigned short uni_k_iroha[];
 #endif
 extern const unsigned char uni_t_x201[];
+extern const unsigned short uni_k_radical[];
 
-extern unsigned short uni_f_nttglyph[]; /* doc*mo keitai glyphs	   */
 extern const unsigned short uni_ibm_nec_excg[]; /* cp932 hooks	   */
 
 /* --- !DYNAMIC LOADING additional includes			   */
@@ -298,15 +304,17 @@ extern unsigned short uni_f_gb2312[];
 extern unsigned short KEISOUT1[];
 extern const unsigned char KEISOUT3[];
 
+/* --------------------------------------------------------------- */
+extern unsigned short x213_rev_conv[];
 /* --------------------------------------------------------------- */
 /* output shift state conditions				   */
 /* --------------------------------------------------------------- */
-extern unsigned int	g0_char,g0_mid,g0_midl; 
-extern unsigned int	ag0_char,ag0_mid,ag0_midl; 
-extern unsigned int	g1_char,g1_mid,g1_midl; 
-extern unsigned int	g2_char,g2_mid,g2_midl; 
-extern unsigned int	g3_char,g3_mid,g3_midl; 
-extern unsigned int	g4_char,g4_mid,g4_midl; 
+extern int		g0_char,g0_mid,g0_midl; 
+extern int		ag0_char,ag0_mid,ag0_midl; 
+extern int		g1_char,g1_mid,g1_midl; 
+extern int		g2_char,g2_mid,g2_midl; 
+extern int		g3_char,g3_mid,g3_midl; 
+extern int		g4_char,g4_mid,g4_midl; 
 extern unsigned long	g0_typ,ag0_typ,g1_typ,g2_typ,g3_typ,g4_typ;
 #define g0_quad		(g0_typ & COD_MB_4)
 #define ag0_quad	(ag0_typ & COD_MB_4)
@@ -481,13 +489,13 @@ extern const unsigned short	o_ascii_entity[];
 	  } else { SKFputc(c3);}; }
 
 /* --------------------------------------------------------------- */
-extern void SKFEUC1OUT P_((skf_ucode));
-extern void SKFEUCOUT P_((skf_ucode));
+extern void SKFEUC1OUT  P_((skf_ucode));
+extern void SKFEUCOUT   P_((skf_ucode));
 extern void SKFEUCG2OUT P_((skf_ucode));
 extern void SKFEUCG3OUT P_((skf_ucode));
 extern void SKFEUCG4OUT P_((skf_ucode));
 
-extern int SKFEUCcount P_((skf_ucode));
+extern int  SKFEUCcount P_((skf_ucode));
 
 /* -------------------------------------------------------------- */
 #define r_SKFEUCOUT(c3) \
@@ -500,11 +508,6 @@ extern int SKFEUCcount P_((skf_ucode));
 	      SKFputc(((c3) & 0xff) | 0x80); }; }
 /* -------------------------------------------------------------- */
 
-extern int  SKFEUCASC	P_(());
-extern int  SKFEUCADD1	P_(());
-extern int  SKFEUCADD2	P_(());
-extern int  SKFEUCADD3	P_(());
-
 /* SJIS */
 #define	SKFSJIS1OUT(c3)	\
 		{ SKFputc(c3);}
@@ -512,30 +515,25 @@ extern int  SKFEUCADD3	P_(());
 #define	SKFSJISK1OUT(c3)	\
 		{ SKFputc((c3) | 0x80U);}
 
-extern void SKFSJISOUT P_((skf_ucode));
+extern void SKFSJISOUT   P_((skf_ucode));
 extern void SKFSJISG2OUT P_((skf_ucode));
 extern void SKFSJISG3OUT P_((skf_ucode));
 extern void SKFSJISG4OUT P_((skf_ucode));
 
-extern int SKFSJIScount P_((skf_ucode));
-
-extern int  SKFSJISASC	P_(());
-extern int  SKFSJISADD1	P_(());
-extern int  SKFSJISADD2	P_(());
-extern int  SKFSJISADD3	P_(());
+extern int SKFSJIScount  P_((skf_ucode));
 
 /* JIS */
-extern void SKFJIS1OUT P_((skf_ucode));
+extern void SKFJIS1OUT	P_((skf_ucode));
 extern void SKFJIS1ASCOUT P_((skf_ucode));
 extern void SKFJISK1OUT P_((skf_ucode));
-extern void SKFJISOUT P_((skf_ucode));
+extern void SKFJISOUT   P_((skf_ucode));
 extern void SKFJISG2OUT P_((skf_ucode));
 extern void SKFJISG3OUT P_((skf_ucode));
 extern void SKFJISG4OUT P_((skf_ucode));
 extern void SKFJIS8859OUT P_((skf_ucode));
 extern void SKFJIS8859XOUT P_((skf_ucode));
 
-extern int SKFJIScount P_((skf_ucode));
+extern int  SKFJIScount P_((skf_ucode));
 
 /* -------------------------------------------------------------- */
 #define r_SKF_ADD_RENEW \
@@ -564,34 +562,24 @@ extern int SKFJIScount P_((skf_ucode));
       SKFputc(p); SKFputc(q); };
 /* -------------------------------------------------------------- */
 /* --- KEIS -------- */
-extern void	SKFKEISOUT P_((skf_ucode));
-extern void	SKFKEIS1OUT P_((skf_ucode));
+extern void	SKFKEISOUT   P_((skf_ucode));
+extern void	SKFKEIS1OUT  P_((skf_ucode));
 extern void	SKFKEISG2OUT P_((skf_ucode));
 extern void	SKFKEISG3OUT P_((skf_ucode));
 extern void	SKFKEISG4OUT P_((skf_ucode));
 extern void 	SKFKEISK1OUT P_((skf_ucode));
-extern void	SKFKEISEOUT P_((skf_ucode));
+extern void	SKFKEISEOUT  P_((skf_ucode));
 
 /* --- BG -------- */
-extern void	SKFBGOUT P_((skf_ucode));
-extern void	SKFBG1OUT P_((skf_ucode));
-extern void	SKFGB2KAOUT P_((skf_ucode));
+extern void	SKFBGOUT     P_((skf_ucode));
+extern void	SKFBG1OUT    P_((skf_ucode));
+extern void 	SKFGB2KAOUT  P_((skf_ucode));	/* gb18030 */
 
 /* --- BRGT -------- */
 extern void	SKFBRGTOUT P_((skf_ucode));
 extern void	SKFBRGT1OUT P_((skf_ucode));
-extern void	SKFBRGTG2OUT P_((skf_ucode));
-extern void	SKFBRGTG3OUT P_((skf_ucode));
-extern void	SKFBRGTG4OUT P_((skf_ucode));
-extern void	SKFBRGTEOUT P_((skf_ucode));
 extern void	tron_announce ();
 
-/* --- Unicode -------- */
-extern int  SKFUNIASC	P_(());
-extern int  SKFUNIADD1	P_(());
-extern int  SKFUNIADD2	P_(());
-extern int  SKFUNIADD3	P_(());
-
 /* ---- string handlers ---------- */
 extern void	SKFJISSTROUT P_((const char *));
 extern void	SKFEUCSTROUT P_((const char *));
@@ -602,6 +590,7 @@ extern void	SKFBRGTSTROUT P_((const char *));
 extern void	SKFBGSTROUT P_((const char *));
 
 extern void	SKFSTROUT P_((const char *));
+extern void	SKFNSTROUT P_((char *, int));
 
 /* ---- mime pre-process --------------- */
 /* Note: BRGT does not handle MIME encoding */
@@ -652,6 +641,8 @@ extern void mime_limit_add P_((int));
 /* ---- line end common routines --- */
 extern void	SKFCRLF();
 
+extern int	sgbuf_buf;
+
 /* ---- output converters --------- */
 #define o_ascii_conv(x)	{if (is_jiscat(conv_cap)) { \
 	    if (is_jis(conv_cap)) JIS_ascii_oconv(x); \
@@ -712,12 +703,11 @@ extern void	print_announce P_((int));
 extern void	cjk_compat_parse P_((skf_ucode));/* compatible plane conv */
 extern void	symbol_conv P_((skf_ucode));	/* 0x2000-0x2dff  */
 extern void	viqr_convert P_((skf_ucode));
-extern void	lig_x0213_out P_((skf_ucode));
+extern void	lig_x0213_out P_((skf_ucode,skf_ucode));
 extern void	lig_compat P_((skf_ucode));
 extern void	GRPH_lig_conv P_((skf_ucode));
 extern void	enc_alpha_supl_conv P_((skf_ucode));
-
-extern int	y_in_dec P_((int));	/* in in_converter.c	  */
+extern void	enc_cjk_supl_conv P_((skf_ucode));
 
 extern int 	test_out_char P_((skf_ucode));
 
@@ -725,6 +715,7 @@ extern void	show_lang_tag ();
 
 extern void	SKF_STRPUT P_((unsigned short *));
 extern void	utf8_uriout P_((skf_ucode));
+extern void	utf8_urioutstr P_((skf_ucode));
 
 extern int	hex_conv_table[];
 
@@ -763,9 +754,10 @@ extern int	skf_lastcount P_((skf_ucode));
 
 #ifdef UNI_DECOMPOSE
 extern void	unicode_normalize_setup();
-extern void	decompose_code P_((unsigned short));
+extern void	decompose_code P_((int));
 extern void	decompose_hangul P_((skf_ucode));
 extern int	get_combine_strength P_((skf_ucode));
+extern void	debugcharout P_((int));;
 
 #define NKD_STRMAP_STRMASK	0x00ffU
 
@@ -781,7 +773,7 @@ extern int	get_combine_strength P_((skf_ucode));
 #define ENCOMPOSE_UPLIMIT	0x20000
 #define ENCOMPOSE_MASK_LEN	512	/* ENCOMPOSE_UPLIMIT/256 */		
 
-#if 0
+#if defined(FUTURESUPPORT)
 /* encompose: encompose is always enabled because of kana */
 struct encompose_ctbl {
     skf_ucode c_identity;	/* candidate char		*/
