@@ -23,7 +23,7 @@
 **	arising by using whole or a part of this software.
 **********************************************************************
     out_encoder.c	output-side converter encode routines
-    $Id: out_encoder.c,v 1.25 2015/04/26 15:08:38 seiji Exp seiji $
+    $Id: out_encoder.c,v 1.35 2019/06/28 16:32:03 seiji Exp seiji $
 
     Punycode encode part is copyrighted by The Internet society.
 */
@@ -58,16 +58,16 @@
 /* o_encode_pend: Encoding temporally suspend. used when
  * no_early_mime_out is disabled.
  */
-	int	o_encode_pend = 0;
+static 	int	o_encode_pend = 0;
 /* o_encode_lc: character count which o_c_encode has received
  * 		plus header/tail length.
  * o_encode_lm: character count which has been thrown to stdout	   */
-	int	o_encode_lc = 0;   /* line length counter	   */
-	int	o_encode_lm = 0;   /* line length counter	   */
+static 	int	o_encode_lc = 0;   /* line length counter	   */
+static 	int	o_encode_lm = 0;   /* line length counter	   */
 #ifdef SUPERFOLD
 static	int	o_encode_lc_save = 0;   /* line pos just before MIME */
 #endif
-	skf_ucode o_encode_max = 0;  /* line character upper limit */
+static 	skf_ucode o_encode_max = 0;  /* line character upper limit */
 static	int	pre_ch = 0;	   /* previous character	   */
 
 static int b64_encoder_table[64] = {
@@ -84,7 +84,6 @@ static int b64_encoder_table[64] = {
 static int b64_encoder_res = 0;	/* encode residuals		  */
 static int b64_encoder_cnt = 0;	/* encode counters		  */
 
-static void show_encode_codeset();
 static void mime_header_gen P_((int));
 static void mime_tail_gen P_((int));
 static void show_encode_codeset P_((int));
@@ -95,7 +94,7 @@ static void char2hex P_((int));
 static void output_to_mime P_((int,int));
 static void queue_to_mime P_((int));
 
-void SKFrCRLF P_(());
+static void SKFrCRLF P_(());
 
 static void base64_enc P_((skf_ucode,int));
 static void mimeq_enc P_((int,int));
@@ -187,21 +186,21 @@ int	offset;
 
 #define enc_pre_qreset() { enc_pre_q_w_ptr = 0; enc_pre_q_r_ptr = 0;}
 
-int enc_pre_qfull()
+static int enc_pre_qfull()
 {
     if (enc_pre_q_w_ptr < O_PREQLIMIT) return(0);
     else return(1);
 }
 
 /* --------------------------------------------------------------- */
-void enc_pre_enque(ch)
+static void enc_pre_enque(ch)
 int ch;
 {
     encode_pre_queue[enc_pre_q_w_ptr++] = ch;
     if (enc_pre_q_w_ptr == O_PREQDEPTH) enc_pre_q_w_ptr = 0;
 }
 
-int enc_pre_deque()
+static int enc_pre_deque()
 {
     int ch;
     if (enc_pre_qempty()) return(sEOF);
@@ -212,7 +211,7 @@ int enc_pre_deque()
 
 #ifdef ACE_SUPPORT
 /* --------------------------------------------------------------- */
-/* internationalized IDN decoders				   */
+/* internationalized IDN decoders(RFC3492)			   */
 /* --------------------------------------------------------------- */
 /* for punycode encoder/decoder --- */
 #define encode_digit(d)	((char)(d + 22 + 75 * (d < 26)))
@@ -220,8 +219,8 @@ int enc_pre_deque()
 /*** Main decode function ***/
 
 /* --- Punycode (RFC 3492) --- */
-/*@+boolint@*/
-int punycode_encode(input_length,input,output_length,output)
+/*@-looploopbreak@*/
+static int punycode_encode(input_length,input,output_length,output)
   int input_length;
   const skf_ucode input[];
   int *output_length;
@@ -232,7 +231,7 @@ int punycode_encode(input_length,input,output_length,output)
 
 #ifdef SKFDEBUG
   if (is_vvv_debug) {
-     fprintf(stderr,"-pe");
+     fprintf(stderr,"-pe(%d,%d)",input_length,*output_length);
   };
 #endif
 
@@ -312,6 +311,7 @@ int punycode_encode(input_length,input,output_length,output)
 /* --------------------------------------------------------------- */
 /* canonical codeset dumper					   */
 /* --------------------------------------------------------------- */
+/*@-branchstate@*/
 void show_encode_codeset(oo_codeset)
 int oo_codeset;
 {
@@ -322,8 +322,10 @@ int oo_codeset;
 
     if ((oo_codeset == codeset_utf16le) 
      || (oo_codeset == codeset_utf16be)
-     || (oo_codeset == codeset_utf16lenb)
-     || (oo_codeset == codeset_utf16lenb)) {
+     || (oo_codeset == codeset_utf16leb)
+     || (oo_codeset == codeset_utf16beb)
+     || (oo_codeset == codeset_nyukan_utf16)
+     || (oo_codeset == codeset_utf16)) {
 	if (o_add_bom) {
 	    desc = "utf-16";
 	} else if (out_endian(conv_cap)) { /* big */
@@ -331,8 +333,11 @@ int oo_codeset;
 	} else {
 	    desc = "utf-16le"; 	
 	};
-    } else if ((oo_codeset == codeset_utf32lenb) 
-     || (oo_codeset == codeset_utf32benb)) {
+    } else if ((oo_codeset == codeset_utf32le) 
+     || (oo_codeset == codeset_utf32be)
+     || (oo_codeset == codeset_utf32leb)
+     || (oo_codeset == codeset_utf32beb)
+     || (oo_codeset == codeset_utf32)) {
 	if (o_add_bom) {
 	    desc = "utf-32";
 	} else if (out_endian(conv_cap)) { /* big */
@@ -349,15 +354,16 @@ int oo_codeset;
 	} else if (oo_codeset == codeset_cp5022x) {
 	    desc = "iso-2022-jp";
 	} else;
+    } else if (desc == NULL) {desc = "";
     } else;
 
-    for (i=0; i< 32; i++, desc++) {
+    for (i=0; i< MIME_CNAME_SIZE; i++, desc++) {
 	if (*desc == '\0') break;
 	SKFcputc(SKFtoupper(*desc));
     };
 }
 
-#if 0
+#if defined(FUTURESUPPORT)
 /* reserved for future use */
 /* --------------------------------------------------------------- */
 int is_idnspace(ch)	/* rfc3454 compliant space detection	   */
@@ -475,15 +481,11 @@ int	state;
 	mime_tail_gen(encode);
 	if (state) {
 	    SKFrCRLF();
-	    if (is_keis(conv_cap)) {
-	    	SKFcputc(KEIS_SP);
-	    } else  SKFcputc(A_SP);
+	    SKFcputc(A_SP);
 	    o_encode_lc = 1;
 	    mime_header_gen(encode);
 	    o_encode_stat = TRUE;
-	} else {
-	    o_encode_stat = FALSE;
-	};
+	} else;
     } else if (is_o_encode_b64(encode)) {
 	SKFrCRLF();
     } else if (is_o_encode_q(encode)) {
@@ -496,7 +498,7 @@ int	state;
 /* --------------------------------------------------------------- */
 /* --- line end handler ------------------------------------------ */
 /* --------------------------------------------------------------- */
-void SKFrCRLF()
+static void SKFrCRLF()
 {
 #ifdef SKFPDEBUG
     if (is_vv_debug) {
@@ -538,15 +540,6 @@ int hex_conv_table[16] = {
 void char2hex(ch)
 int	ch;
 {
-    if (ch >= 0x10000) {
-	SKFcputc(hex_conv_table[(ch >> 20) & 0x0fU]);
-	SKFcputc(hex_conv_table[(ch >> 16) & 0x0fU]);
-	ch &= 0xffffU;
-    };
-    if (ch >= 0x100) {
-	SKFcputc(hex_conv_table[(ch >> 12) & 0x0fU]);
-	SKFcputc(hex_conv_table[(ch >> 8) & 0x0fU]);
-    };
     SKFcputc(hex_conv_table[(ch >> 4) & 0x0fU]);
     SKFcputc(hex_conv_table[(ch & 0x0fU)]);
 }
@@ -555,16 +548,6 @@ void char2oct(ch)
 /*@unused@*/
 int ch;
 {
-    if (ch >= 0x10000) {
-	SKFcputc('0'+((ch >> 22) & 0x03U));
-	SKFcputc('0'+((ch >> 19) & 0x07U));
-	SKFcputc('0'+((ch >> 16) & 0x07U));
-    };
-    if (ch >= 0x100) {
-	SKFcputc('0'+((ch >> 14) & 0x03U));
-	SKFcputc('0'+((ch >> 11) & 0x07U));
-	SKFcputc('0'+((ch >> 8) & 0x07U));
-    };
     SKFcputc('0'+((ch >> 6) & 0x03U));
     SKFcputc('0'+((ch >> 3) & 0x07U));
     SKFcputc('0'+(ch & 0x07U));
@@ -591,11 +574,7 @@ int	encode;
 	fprintf(stderr,",#%d,%x)",b64_encoder_cnt,b64_encoder_res);
     };
 #endif
-    if (ch >= 0x100) {
-	out_undefined(ch,SKF_ENC_ERR);
-	b64_encoder_cnt = 0;
-	b64_encoder_res = 0;	/* for extra care		   */
-    } else if (ch < 0) {
+    if (ch < 0) {
 	if (b64_encoder_cnt == 2) {
 	    c3 = (b64_encoder_res << 2) & 0x03cU;
 	    c4 = b64_encoder_table[c3];
@@ -714,6 +693,7 @@ int o_encode;
     if (is_vv_debug) fprintf(stderr,"QM");
 #endif
 
+/*@-infloopsuncon@*/
     while (!enc_pre_qempty()) {
     	ch = enc_pre_deque();
 	if (ch < 0) continue;
@@ -723,6 +703,7 @@ int o_encode;
 	    output_to_mime(ch,o_encode);
 	};
     };
+/*@+infloopsuncon@*/
 }
 
 #ifdef ACE_SUPPORT
@@ -797,11 +778,13 @@ skf_ucode ch;
 /*@+infloopsuncon@*/
 #endif
 
+#ifdef FUTURESUPPORT
 /* --------------------------------------------------------------- */
 /* showing mime-specified charset 				   */
 /* return value:						   */
 /*	0:  parse succeeded 					   */
 /*	-1: failed to parse, or undecodable charset		   */
+/*  reserved for future expansion				   */
 /* --------------------------------------------------------------- */
 
 int show_mime_charset(sy)
@@ -832,6 +815,7 @@ int	*sy;
     in_codeset = sss;
     return(0);
 }
+#endif
 
 /* --------------------------------------------------------------- */
 /* encoder main(0) - clip test.					   */
@@ -847,6 +831,7 @@ int	*sy;
 /* --------------------------------------------------------------- */
 /* no clip: ace, uri, hex, oct					   */
 /* --------------------------------------------------------------- */
+/*@-infloopsuncon@*/
 int mime_clip_test(offset,soffset)
 int offset;
 int soffset;
@@ -941,6 +926,7 @@ int soffset;
 	};
     } else if (is_o_encode_mimeq(o_encode) || is_o_encode_q(o_encode)) {
 	chlen = offset + (soffset * 3) + mime_max_chlen + prequeue_depth;
+	if (is_o_encode_q(o_encode)) chlen--;
     	if (o_encode_stat == 0) { /* encoding not started	   */
 	    if (((o_encode_lc + chlen) >= mime_start_limit) ||
 	    	(soffset != 0) || (is_ucs_utf16(conv_cap))) {
@@ -1000,6 +986,7 @@ int soffset;
     } else;
     return(0);
 }
+/*@+infloopsuncon@*/
 
 /* --------------------------------------------------------------- */
 /* encoder main(2) - character encoding				   */
@@ -1034,7 +1021,7 @@ int	ch;
 	else if (ch == sSUSP) fprintf(stderr,"sSUSP");
 	else if (ch == sFLSH) fprintf(stderr,"sFLSH");
 	else if (ch == mFLSH) fprintf(stderr,"mFLSH");
-	else fprintf(stderr,"%x",ch);
+	else fprintf(stderr,"%02x",ch);
 	fprintf(stderr,"(%d/%d-%d)",o_encode_lm,o_encode_lc,
 		prequeue_depth);
 	if (ch >= 0) {
@@ -1109,11 +1096,7 @@ int	ch;
 		    queue_to_mime(o_encode);
 		};
 	    } else;
-#if 0
-	    if ((ch == sFLSH) || (ch == mFLSH) || (ch == sSUSP)) {
-#else
 	    if ((ch == sFLSH) || (ch == mFLSH)) {
-#endif
 		    /* puny length limit is considered in other */
 		    /* routine. Here we test only MIMEs	   */
 		queue_to_mime(o_encode);
@@ -1261,7 +1244,9 @@ void puny_finish_procedure()
 
     oconv_flush();
     if ((o_encode_stat == 0) || (puny_maxcp == 0)) {
+    /*@-infloopsuncon@*/
 	while (!enc_pre_qempty()) SKFcputc(enc_pre_deque());
+    /*@+infloopsuncon@*/
 	o_encode_stat = FALSE;
 	enc_pre_qreset();
 	puny_maxcp = 0;
