@@ -23,7 +23,7 @@
 **********************************************************************
 ** plug_convert.c:	various converter routines for skf
 **		to treat something dirty ;-)
-** $Id: plug_convert.c,v 1.116 2014/12/26 16:32:07 seiji Exp seiji $
+** $Id: plug_convert.c,v 1.124 2019/04/06 13:16:54 seiji Exp seiji $
 **	x0201conv:	x0201 kana converter
 **	new2oldJis:	x0208(1983,1990) to C6226(1978)
 **	lig_x0213_out:	X0213/ISCII ligature out 
@@ -64,8 +64,8 @@ static void	CJK_display_day P_((skf_ucode));
 
 static void	CJK_circled P_((skf_ucode,int));
 
-       void	lig_compat P_((skf_ucode));
 static void	kana_force_conv P_((skf_ucode));
+static void	utf32_to_hex P_((skf_ucode, int));
 
 /* ---------------------------------------------------------------
  *   SKF_STRPUT():	short binary string output
@@ -116,7 +116,8 @@ int x0201conv(c,nxt)
 
 #ifdef SKFDEBUG
 	if (is_vv_debug)
-		fprintf(stderr," x0201conv: %x-%x",c,nxt);
+		fprintf(stderr," x0201conv: %x-%x",
+			(unsigned int)c,(unsigned int)nxt);
 #endif
 	c3 = (c & 0x00ff) - 0x20;
 	unxt = nxt & 0x7f;		/* equalization		   */
@@ -190,7 +191,7 @@ skf_ucode ch;
 
 #ifdef SKFDEBUG
     if (is_vv_debug)
-		fprintf(stderr," x0201rconv:%x",ch);
+	fprintf(stderr," x0201rconv:%x",(unsigned int)ch);
 #endif
     if ((ch >= 0x3041) && (ch <= 0x3096)) {
     	return(kana_revcnv[ch - 0x3041]);
@@ -230,6 +231,11 @@ skf_ucode ch;
 /* various ligatures and special treatments			   */
 /* --------------------------------------------------------------- */
 /* lig_x0213_out: input - 0xd800 - 0xdfff			   */
+/*   d800-d87f: x0213 kanas/iscii (6b-6f: reserved)		   */
+/*   d880-d8ff: tiscii 						   */
+/*   d900-d9cf: mosaics						   */
+/*   da00-dacf: mosaics						   */
+/*   db00-db8f: tscii(2)					   */
 /* --------------------------------------------------------------- */
 
 static unsigned short uni_lig_emo_f9[16] = {
@@ -245,17 +251,45 @@ static skf_ucode uni_lig_emo_fa[20] = {
     0x1f1f7,0x1f1f3, 0x1f1f7,0x1f1f8
 };
 
+static const char *uni_lig_arib_mi[32] = {
+  "(vn)","(ob)","(cb)","(ce",  "mb)","(hp)","(br)","(p)",
+  "(s)","(ms)","(t)","(bs)",   "(b)","(tb)","(tp)","(ds)",
+  "(ag)","(eg)","(vo)","(fl)", "(ke","y)","(sa","x)",
+  "(sy","n)","(or","g)",       "(pe","r)"," "," "
+};
+
 #ifdef TSCII_SUPPORT
-static unsigned short *uni_lig_ti;
+static const unsigned short tscii_bct[22] = {
+    0xb8, 0xb9, 0xba, 0xbb,  0xbc, 0xbd, 0xbe, 0xbf,
+    0xc0, 0xc1, 0xc2, 0xc3,  0xc4, 0xc5, 0xc6, 0xc7,
+    0xc8, 0xc9, 0x83, 0x84,  0x85, 0x86
+};
+
+static const unsigned short tscii_blig[66] = {
+  0xec, 0xed, 0xee, 0xef, 0xf0, 0xf1, 0xf2, 0xf3,
+  0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0xfa, 0xfb,
+  0xfc, 0xfd, 0x88, 0x89, 0x8a, 0x8b,
+  0xcc, 0x99, 0xcd, 0x9a, 0xce, 0xcf, 0xd0, 0xd1,
+  0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9,
+  0xda, 0xdb, 0x00, 0x00, 0x00, 0x00,
+  0xdc, 0x9b, 0xdd, 0x9c, 0xde, 0xdf, 0xe0, 0xe1,
+  0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9,
+  0xea, 0xeb, 0x00, 0x00, 0x00, 0x00
+};
 #endif
 
-void lig_x0213_out(ch)
+void lig_x0213_out(ch,rch)
 skf_ucode ch;
+skf_ucode rch;
 {
     int	c1,c2;
     int	cn;
+#ifdef TSCII_SUPPORT
+    int v1,v2,v3;
+#endif
 #ifdef SKFDEBUG
-    if (is_vv_debug) fprintf(stderr," x0213lig(%x)",ch);
+    if (is_vv_debug) fprintf(stderr," x0213lig(%x:%x)",
+    			(unsigned short)ch,(unsigned short)rch);
 #endif
     if (ch < 0xd880) {
 	c1 = (ch & 0x7f);	/* U+d801 - U+d87f		   */
@@ -263,16 +297,23 @@ skf_ucode ch;
 	    c2 = uni_lig_x0213_24[c1];	/* 0x24xx, 0x25xx	   */
 	    if (c2 != 0) {
 		post_oconv(c2);
-		if (!is_ms_213c(conv_cap) && !is_euc_213c(conv_cap)
-		    	 && !is_jis_213c(conv_cap) 
-			 && !is_nkf_compat
-		     && (in_codeset != codeset_x0213)
-		     && (in_codeset != codeset_x213a)
-		     && (in_codeset != codeset_x0213_s)
-		     && (in_codeset != codeset_euc_213)
-		     && (in_codeset != codeset_euc_213a)
-		     && (in_codeset != codeset_sj_0213)
-		     && (in_codeset != codeset_sj_213a)
+		if (is_ucs_ufam(conv_cap)) {
+		    if (((is_x0213(g0_table_mod->is_kana)) ||
+		         (is_x0213(g1_table_mod->is_kana)) ||
+		         (is_x0213(g2_table_mod->is_kana)) ||
+		         (is_x0213(g3_table_mod->is_kana))) &&
+			 (rch == 0)) {
+			post_oconv(0x309a);
+		    } else if (rch == 0) post_oconv(0x309c);
+		    else post_oconv(rch);
+		} else if (!is_ms_213c(conv_cap) 
+			&& !is_euc_213c(conv_cap)
+		    	&& !is_jis_213c(conv_cap) 
+			&& !is_nkf_compat
+		     && (!is_x0213(g0_table_mod->is_kana))
+		     && (!is_x0213(g1_table_mod->is_kana))
+		     && (!is_x0213(g2_table_mod->is_kana))
+		     && (!is_x0213(g3_table_mod->is_kana))
 			) {
 		    post_oconv(0x309c);
 		} else {
@@ -343,51 +384,51 @@ skf_ucode ch;
 		out_undefined(ch,SKF_UNSUPP);
 	    };
 	};
+    } else if (ch < 0xd9d0) {	/* mosaics (0xd900 - 0xd9cf)	   */
+    	/* do everything before this point.			   */
+	out_undefined(ch,SKF_IOUTUNI);
+    } else if (ch < 0xda00) {	/* mosaics (0xd9d0 - 0xd9ff)	   */
+	if (ch < 0xd9f0) {
+	    SKFSTROUT(uni_lig_arib_mi[ch - 0xd9d0]);
+    	} else out_undefined(ch,SKF_IOUTUNI);
 #ifdef TSCII_SUPPORT
-    } else if (ch < 0xd900) {	/* tscii 1.7			   */
-    	if ((uni_lig_ti == NULL) || (uni_lig_tv == NULL)) {
-	    if ((uni_lig_ti = load_external_table(
-	    	  &(ovlay_table_defs[tscii_i_index])) < 0)) {
-		out_undefined(ch,SKF_NOTABLE);
-		return;
-	    };
-	};
-	c1 = ch & 0x7f;
-	if (uni_lig_tcsii[c1+128]) {
-	    post_oconv(uni_lig_tcsii[c1+128]);
-	} else {
-	    out_undefined(ch,SKF_UNSUPP);
-	};
-	if (uni_lig_tscii[c1]) {
-	    post_oconv(uni_lig_tcsii[c1]);
-	    if (c1 == 0x82) {
-		post_oconv(0x0bb0);
-		post_oconv(0x0bc0);
-	    } else if (c1 == 0x87) {
-		post_oconv(0x0bb7);
-#ifdef FOLD_SUPPORT
-	    fold_count--;
-#endif
-	    } else if (c1 == 0x8c) {
-		post_oconv(0x0bb7);
-		post_oconv(0x0bcd);
-	    };
-#ifdef FOLD_SUPPORT
+    } else if (ch < 0xdb42) {	/* tscii two-part vowels	   */
+    	if (is_tscii(conv_cap)) {
+	    v1 = (ch < 0xdb16) ? 0xa6 :
+		    ((ch < 0xdb2c) ? 0xa7 : 0xa6);
+	    v2 = tscii_bct[(ch - 0xdb00) % 22];
+	    v3 = (ch < 0xdb2c) ? 0xa1 : 0xaa;
+	    SKFputc(v1);
+	    SKFputc(v2);
+	    SKFputc(v3);
 	    fold_count += 3;
+	    return;
+	} else;
+    } else if (ch < 0xdb84) {	/* tscii Mey, Ukara, Uukaara	   */
+    	if (is_tscii(conv_cap)) {
+	    v1 = tscii_blig[ch - 0xdb42];
+	    SKFputc(v1);
+	    fold_count += 1;
+	    return;
+	} else;
+    } else if (ch < 0xdb8a) {	/* tscii two-part vowels	   */
+        if (is_tscii(conv_cap)) {
+	    switch(ch) {
+		case 0xdb84: SKFputc(0xca); break;
+		case 0xdb85: SKFputc(0xcb); break;
+		case 0xdb86: SKFputc(0x87); break;
+		case 0xdb87: SKFputc(0x8c); break;
+		case 0xdb88: SKFputc(0x82); break;
+		case 0xdb89: SKFputc('.'); 
+			fold_count++;/*@FALLTHROUGH@*/
+		default: SKFputc('.'); SKFputc('.'); break;
+	    };
+	    fold_count += 2;
+	    return;
+	} else;
 #endif
-	} else {
-	    out_undefined(ch,SKF_UNSUPP);
-#ifdef FOLD_SUPPORT
-	    fold_count += 3;
-#endif
-	};
-#endif
-    } else if (ch < 0xd9d0) {	/* mosaics (0xd900 - 0xd9df)	   */
-    	/* do anything before this point.			   */
-	out_undefined(ch,SKF_IOUTUNI);
-    } else {
-	out_undefined(ch,SKF_IOUTUNI);
-    };
+    } else out_undefined(ch,SKF_IOUTUNI);
+    return;
 }
 
 /* --------------------------------------------------------------- */
@@ -457,7 +498,7 @@ skf_ucode c1;
     sym = NULL;
     c2 = (c1 & 0x00ff);
 #ifdef SKFDEBUG
-    if (is_vv_debug) fprintf(stderr," ligature at %x",c1);
+    if (is_vv_debug) fprintf(stderr," ligature at %x",(unsigned short)c1);
 #endif
     if ((c2 >= 0x58) && (c2 <= 0x70)) {
 	CJK_display_ten(c2);
@@ -523,18 +564,22 @@ skf_ucode c1;
 /* --------------------------------------------------------------
     CJK_circled:	circled CJK character pseudo converter
 	c1:	output character
-	c2: bit 3: output as is
+	c3: bit 3: output as is
 	    bit 2: output small letters
 	    bit 1: output capital letters
 	    bit 3,2,1 = 0: output digits
 	    bit 0 = 1: output dot
-	    bit 0 = 0: output bracket
+	    bit 0 = 0 and bit 4 = 0: output bracket
+	    bit 0 = 0 and bit 4 = 1: output squared bracket
 */
 void CJK_circled(c1,c3)
 skf_ucode c1;
 int c3;
 {
-    if ((c3 & 0x01) == 0) { post_oconv('('); };
+    if ((c3 & 0x01) == 0) {
+    	if (c3 & 0x10) post_oconv('[');
+	else post_oconv('('); 
+    } else;
 
     if ((c3 & 0x02) != 0) post_oconv(c1+'A');
     else if ((c3 & 0x04) != 0) post_oconv(c1+'a');
@@ -542,7 +587,8 @@ int c3;
     else cjk_number_parse(c1);
 
     if ((c3 & 0x01) == 0) {
-	post_oconv(')'); 
+    	if (c3 & 0x10) post_oconv(']');
+	else post_oconv(')');
     } else { 
 	post_oconv('.');
     };
@@ -566,17 +612,23 @@ skf_ucode c2;
     sym = NULL;
     c1 = c2 & 0xff;
     if ((c1 >= 0x20) && (c1 <= 0x43)) {
-	c1 -= 0x20; post_oconv(0x28);
-	post_oconv(uni_k_enl[c1]); 
-	post_oconv(0x29);
+	CJK_circled(uni_k_enl[c1 - 0x20],0x8);
+    } else if ((c1 >= 0x48) && (c1 <= 0x4f)) {
+	post_oconv(0x5b);
+	post_oconv(c1 - 0x47+0x30); post_oconv(0x30);
+	post_oconv(0x5d);
     } else if ((c1 >= 0x80) && (c1 <= 0x98)) {
-	c1 -= 0x80; post_oconv(0x28);
-	post_oconv(uni_k_enl[c1]);
-	post_oconv(0x29);
+	CJK_circled(uni_k_enl[c1 - 0x80],0x8);
     } else if ((c1 >= 0x99) && (c1 <= 0xb0)) {
-	c1 -= 0x99; post_oconv(0x28);
-	post_oconv(uni_k_cil[c1]);
-	post_oconv(0x29);
+	CJK_circled(uni_k_cil[c1 - 0x99],0x8);
+    } else if (c1 == 0x44) {		/* toi			*/
+    	CJK_circled(0x554f,0x8);
+    } else if (c1 == 0x45) {		/* you			*/
+    	CJK_circled(0x5e7c,0x8);
+    } else if (c1 == 0x46) {		/* bun			*/
+    	CJK_circled(0x6587,0x8);
+    } else if (c1 == 0x47) {		/* sho			*/
+    	CJK_circled(0x7b8f,0x8);
     } else if (c1 == 0x50) {		/* PTE			*/
 	sym = "PTE";
     } else if ((c1 >= 0x51) && (c1 <= 0x5f)) { /* Enclosed num */
@@ -598,7 +650,7 @@ skf_ucode c2;
     } else if (c1 == 0xff) {	/* new era ligature		*/
     	post_oconv(0x4ee4);
     	post_oconv(0x548c);
-    } else {		  /* just undefied 		   */
+    } else {			/* just undefined 		*/
 	out_undefined(c2,SKF_OUNDEF);
     };
     if (sym != NULL) {
@@ -736,11 +788,14 @@ skf_ucode c4;
     skf_ucode	u0;
     int		mod;
 
+#ifdef SKFDEBUG
+    if (is_vv_debug) fprintf(stderr,"(Lig)");
+#endif
     c1 = (skf_ucode)(c4 & 0xffUL);
     c2 = (skf_ucode)((c4 & 0xff00UL) >> 8);
 
     if (c2 == 0x20) {
-	if (is_keis(conv_cap) &&
+	if (is_keis(conv_cap) &&	/* currently not used */
 		((c1 == 0x20) || (c1 == 0x21))) {
 	    SKFKEISEOUT(0x7fcd + c1);
 	} else {
@@ -760,6 +815,8 @@ skf_ucode c4;
 	    ostr = uni_f_s_21[c1];
 	    if (ostr != NULL) SKFSTROUT(ostr);
 	    else out_undefined(c4,SKF_OUNDEF);
+	} else if (c1 == 0x89) {
+	    SKFSTROUT("0/3");
 	} else {
 	    out_undefined(c4,SKF_OUNDEF);
 	};
@@ -798,6 +855,7 @@ skf_ucode c4;
 	    out_undefined(c4,SKF_OUNDEF);
 	};
     } else if (c2 == 0x25) {  /* Form and Chart components.   */
+#ifdef JEF_OUTPUT
 	if (is_keis_jef(conv_cap) &&
 	    ((c1 == 0x25) || (c1 == 0x30) || (c1 == 0x1d)
 			|| (c1 == 0x38) || (c1 == 0xef))) {
@@ -812,13 +870,23 @@ skf_ucode c4;
 		post_oconv(u0); 
 	    } else out_undefined(c4,SKF_OUNDEF); 
 	};
+#else
+	if ((c1 <= 0x7f) && (skf_is_ja(output_lang))) { 
+	    u0 = moji_kei[c1];
+	    post_oconv(u0); 
+	} else out_undefined(c4,SKF_OUNDEF); 
+#endif
     } else if (c2 == 0x26) {  /* Misc. dingbats		   */
+#ifdef JEF_OUTPUT
 	if (is_keis_jef(conv_cap) && 
 	    ((c1 == 0x6f) || (c1 == 0x6d) || (c1 == 0x6a))) {
 	    if (c1 == 0x6a) SKFKEISEOUT(0x7fec);
 	    else if (c1 == 0x6d) SKFKEISEOUT(0x7feb);
 	    else SKFKEISEOUT(0x7fea);
 	} else out_undefined(c4,SKF_OUNDEF);
+#else
+	out_undefined(c4,SKF_OUNDEF);
+#endif
     } else if (c2 == 0x27) {  /* Zapf dingbats		   */
 	if ((c1 >= 0x76) && (c1 <= 0x7f)) {
 	    CJK_circled(c1 - 0x75,0);
@@ -838,6 +906,27 @@ skf_ucode c4;
 	out_undefined(c4,SKF_OUNDEF);
     } else if (c2 == 0x2a) {  /* Suppremental Math Operators   */
 	out_undefined(c4,SKF_OUNDEF);
+    } else if (c2 == 0x2b) {  /* misc symbols		   */
+    	if (c1 == 0x1b) post_oconv((skf_ucode)(0x25a0));
+    	else if (c1 == 0x1c) post_oconv((skf_ucode)(0x25a1));
+    	else if (c1 == 0x24) post_oconv((skf_ucode)(0x25cf));
+    	else if (c1 == 0x25) post_oconv((skf_ucode)(0x25c6));
+    	else if (c1 == 0x26) post_oconv((skf_ucode)(0x25c7));
+    	else if (c1 == 0x55) post_oconv((skf_ucode)(0x25ef));
+    	else if (c1 == 0x58) post_oconv((skf_ucode)(0x25ef));
+	else if ((c1 >= 0x60) && (c1 <= 0x69))
+		post_oconv((skf_ucode)(0x2190 + (c1 - 0x60)));
+	else if ((c1 >= 0x6a) && (c1 <= 0x6d))
+		post_oconv((skf_ucode)(0x21e0 + (c1 - 0x6a)));
+	else if ((c1 >= 0x84) && (c1 <= 0x87))
+		post_oconv((skf_ucode)(0x21c7 + (c1 - 0x84)));
+    	else if (c1 == 0xbd) post_oconv((skf_ucode)(0x1f147));
+    	else if (c1 == 0xbe) post_oconv((skf_ucode)(0x24e7));
+    	else if (c1 == 0xbf) post_oconv((skf_ucode)(0x24e7));
+    	else if (c1 == 0xc0) post_oconv((skf_ucode)(0x25a0));
+    	else if (c1 == 0xc5) post_oconv((skf_ucode)(0x25b2));
+    	else if (c1 == 0xc6) post_oconv((skf_ucode)(0x25bc));
+	else out_undefined(c4,SKF_OUNDEF);
     } else {		  /* 0x2b00-0x2dff		   */
 	out_undefined(c4,SKF_OUNDEF);
     };
@@ -877,27 +966,34 @@ skf_ucode c1;
     	input:		c1 - input character
 	output:		none
 */
-static char *u_bra_lig_supl[11] = {
-  "DJ","CL","COOL","FREE","ID","NEW","NG","OK","SOS",
-  "UP!","VS"
+static char *u_bra_lig_supl[28] = {
+  "CL","COOL","FREE","ID",   "NEW","NG","OK","SOS",
+  "UP!","VS","3D","2nd Scr", "2K","4K","8K","5.1",
+  "7.1","22.2","60P","120P", "d","HC","HDR","Hi-res",
+  "Lossless","SHV","UHD","VOD"
 };
 
-void enc_alpha_supl_conv(c1)
+void enc_alpha_supl_conv(c1)	/* 1f100 - 1f200 */
 skf_ucode c1;
 {
     int c2;
 
+#ifdef SKFDEBUG
+    if (is_vvv_debug) {
+    	fprintf(stderr,"-EnSC:%x ",c1);
+    } else;
+#endif
+
     if (c1 < 0x1f110) {
         if (c1 == 0x1f100) {
-	    post_oconv(((skf_ucode)('0')));
-	    post_oconv(((skf_ucode)('.')));
+	    CJK_circled('0',0x09);
 	} else if (c1 <= 0x1f10a) {
 	    post_oconv(((skf_ucode)(c1 - 0x1f101 + 0x30)));
 	    post_oconv(((skf_ucode)(',')));
 	} else {
 	    out_undefined(c1,SKF_OUNDEF);
 	};
-    } else if (c1 < 0x1f190) {
+    } else if (c1 <= 0x1f190) {
     	if (c1 < 0x1f130) {
 	    c2 = c1 - 0x1f110;
 	} else if (c1 < 0x1f150) {
@@ -906,9 +1002,9 @@ skf_ucode c1;
 	    c2 = c1 - 0x1f150;
 	} else c2 = c1 - 0x1f170;
 	if (c2 < 0x1a) {
-	    post_oconv(((skf_ucode)('(')));
-	    post_oconv(((skf_ucode)(c2 + 0x41)));
-	    post_oconv(((skf_ucode)(')')));
+	    CJK_circled((c2 + 0x41),
+	    	(((c1<0x1f130) || ((c1>=0x1f150) && (c1<0x1f170)))
+			? 0x08 : 0x18));
 	} else {
 	    switch(c1) {
 	    	case 0x1f12a: SKFSTROUT("[S]"); break;
@@ -930,19 +1026,76 @@ skf_ucode c1;
 	    	case 0x1f18d: SKFSTROUT("[SA]"); break;
 	    	case 0x1f18e: SKFSTROUT("[AB]"); break;
 	    	case 0x1f18f: SKFSTROUT("[WC]"); break;
+	    	case 0x1f190: SKFSTROUT("DJ"); break;
 	    	default: out_undefined(c1,SKF_OUNDEF);
 	    };
 	};
-    } else if (c1 <= 0x1f1aa) {
-    	SKFSTROUT(u_bra_lig_supl[c1 - 0x1f1a0]);
+    } else if (c1 <= 0x1f1ac) {
+	post_oconv((skf_ucode)('['));
+    	SKFSTROUT(u_bra_lig_supl[c1 - 0x1f191]);
+	post_oconv((skf_ucode)(']'));
     } else if (c1 >= 0x1f1e6) {
-	    post_oconv(((skf_ucode)(c1 - 0x1f1e6 + 0x41)));
+	post_oconv(((skf_ucode)(c1 - 0x1f1e6 + 0x41)));
     } else {
 	out_undefined(c1,SKF_OUNDEF);
     };
 }
+
+static skf_ucode u_bra_lig_cjk_s[55] = {
+    0x30b5,0x624b,0x5b57,0x53cc,0x30c7,0x4e8c,0x591a,0x89e3,
+    0x5929,0x4ea4,0x6620,0x7121,0x6599,0x524d,0x5f8c,0x518d,
+    0x65b0,0x521d,0x7d42,0x751f,0x8ca9,0x58f0,0x5439,0x6f14,
+    0x6295,0x6355,0x4e00,0x4e09,0x904a,0x5de6,0x4e2d,0x53f3,
+    0x6307,0x8d70,0x6253,0x7981,0x7a7a,0x5408,0x6e80,0x6709,
+    0x6708,0x7533,0x5272,0x55b6,0x672c,0x4e09,0x4e8c,0x5b89,
+    0x70b9,0x6253,0x76d7,0x52dd,0x6557,0x5f97,0x53ef
+};
+
+/* ---------------------------------------------------------------
+    enclose_cjk_suppl_conv:	Enclosed CJK supplement conv.
+    			1f200 - 1f2ff
+    	input:		c1 - input character
+	output:		none
+*/
+void enc_cjk_supl_conv(c1)	/* 1f200 - 1f2ff */
+skf_ucode c1;
+{
+    int c2;
+
+#ifdef SKFDEBUG
+    if (is_vvv_debug) {
+    	fprintf(stderr,"-EnSK:%x ",c1);
+    } else;
+#endif
+    c2 = c1 - 0x1f202;
+    if (c1 == 0x1f200) {
+	post_oconv((skf_ucode)('['));
+    	post_oconv((skf_ucode)0x307b);
+    	post_oconv((skf_ucode)0x304b);
+	post_oconv((skf_ucode)(']'));
+	return;
+    } else if (c1 == 0x1f201) {
+	post_oconv((skf_ucode)('['));
+    	post_oconv((skf_ucode)0x30b3);
+    	post_oconv((skf_ucode)0x30b3);
+	post_oconv((skf_ucode)(']'));
+	return;
+    } else if ((c1 >= 0x1f210) && (c1 <= 0x1f23a)) {
+    	c2 = c1 - 0x1f210 + 1;
+    } else if ((c1 >= 0x1f240) && (c1 <= 0x1f248)) {
+    	c2 = c1 - 0x1f240 + 1 + 43;
+    } else if ((c1 >= 0x1f250) && (c1 <= 0x1f251)) {
+    	c2 = c1 - 0x1f250 + 1 + 43 + 9;
+    } else {
+	out_undefined(c1,SKF_OUNDEF);
+	return;
+    };
+    CJK_circled(u_bra_lig_cjk_s[c2],0x18);
+    return;
+}
 /* ---------------------------------------------------------------
     kana_force_conv:	convert miscellaneous kana part
+    			0x3000 - 0x3400 - called from skf_lastresort
     	input:		c2 - input character
 	output:		none
 */
@@ -959,6 +1112,9 @@ skf_ucode c2;
 	    post_oconv((skf_ucode)(','));
 	    post_oconv((skf_ucode)(','));
 	    converted = 1;
+	} else if (c2 == 0x3036){ /* ARIB bracketted post-mark */
+	    CJK_circled(0x3012,0x8);
+	    converted = 1;
 	} else if (c2 == 0x303f){ /* ideographic single space*/
 	    post_oconv((skf_ucode)(A_SP));
 	    converted = 1;
@@ -981,6 +1137,7 @@ skf_ucode c2;
     if (converted == 0)
 	    out_undefined(c2,SKF_OUNDEF);
 }
+/*@+globstate@*/
 
 /* ---------------------------------------------------------------
     print_announce:	print announcer sequence for fj-code
@@ -1068,6 +1225,9 @@ void show_lang_tag()
 	    SKFputc((int)(ll & 0x7f));
 	};
     } else if (is_brgt(conv_cap)) { /* TAD Announcer	   */
+#ifdef SKFDEBUG
+	if (is_vv_debug) fprintf(stderr," bright-ann ");
+#endif
 	tron_announce();
     };
 }
@@ -1090,9 +1250,9 @@ void show_endian_out()
 #endif
 	if (out_endian(conv_cap)) {
 	    SKFputc(0x00); SKFputc(0x00);
-	    SKFputc(0xfe); SKFputc(0xff);
+	    SKFputc(BOM_LOW); SKFputc(BOM_HIGH);
 	} else {
-	    SKFputc(0xff); SKFputc(0xfe);
+	    SKFputc(BOM_HIGH); SKFputc(BOM_LOW);
 	    SKFputc(0x00); SKFputc(0x00);
 	};
       } else {
@@ -1100,9 +1260,9 @@ void show_endian_out()
 	if (is_vv_debug) fprintf(stderr," ucs2-bom\n");
 #endif
 	if (out_endian(conv_cap)) {
-	    SKFputc(0xfe); SKFputc(0xff);
+	    SKFputc(BOM_LOW); SKFputc(BOM_HIGH);
 	} else {
-	    SKFputc(0xff); SKFputc(0xfe);
+	    SKFputc(BOM_HIGH); SKFputc(BOM_LOW);
 	};
       };
     } else if (is_ucs_utf8(conv_cap)) { /* UTF-8		   */
@@ -1401,17 +1561,66 @@ void skf_lastresort(c1)
 skf_ucode c1;
 {
     int enbl = FALSE;
+    int	c2,c3;
 
 #ifdef SKFDEBUG
     if (is_vv_debug) fprintf(stderr,"-LR(%x)",c1);
 #endif
     if (use_latin2html) enbl = latin2html(c1);
     if (use_latin2tex && !enbl) enbl = latin2tex(c1);
-    if ((use_latin2null && is_nkf_compat) 
-    		|| is_no_bfc) return;	/* discard 'em	   */
+    if (use_latin2java && !enbl) {
+	if (!is_valid_ucode(c1)) {
+	    out_undefined(c1,SKF_OUNDEF);
+	    return;
+	} else;
+	SKFputc(A_YEN); 
+    	if (c1 >= 0x10000) {	/* USE utf-16	*/
+	    SKFputc('u'); 
+	    c3 = (((c1 >> 10) - 0x040) & 0x3ffU) + 0xd800;
+	    utf32_to_hex(c3,1);
+	    SKFputc(A_YEN); SKFputc('u'); 
+	    c2 = (int) ((c1 & 0x3ffU) + 0xdc00);
+	    utf32_to_hex(c2,1);
+	} else {
+	    SKFputc('u');
+	    utf32_to_hex(c1,1);
+	};
+    	enbl = TRUE;
+    } else if (use_latin2perl && !enbl) {
+	if (!is_valid_ucode(c1)) {
+	    out_undefined(c1,SKF_OUNDEF);
+	    return;
+	} else;
+	SKFputc(A_YEN); 
+	SKFputc('x'); SKFputc('{');
+	utf32_to_hex(c1,0);
+	SKFputc('}');
+    	enbl = TRUE;
+    } else if (use_latin2python && !enbl) {
+	if (!is_valid_ucode(c1)) {
+	    out_undefined(c1,SKF_OUNDEF);
+	    return;
+	} else;
+	SKFputc(A_YEN); 
+	if (c1 >= 0x10000) {
+	    SKFputc('U'); SKFputc('0'); SKFputc('0');
+	} else SKFputc('u');
+	utf32_to_hex(c1,1);
+    	enbl = TRUE;
+    } else if ((use_latin2null && is_nkf_compat)
+    	|| is_no_bfc) return;	/* discard 'em	   */
 
     if (!enbl) {
-	if ((c1 >= 0x3000) && (c1 <= 0x4e00) &&
+	if ((c1 < 0xa0) ||
+	    (out_codeset == codeset_cp932w)) {
+	 	;	/* do nothing */
+	} else if (c1 < 0x100) {
+	    ascii_fract_conv(c1);
+	    enbl = TRUE;
+	} else if (c1 < 0x3000) {
+	    GRPH_lig_conv(c1);  
+	    enbl = TRUE;
+	} else if ((c1 <= 0x4e00) &&
 	    (out_codeset != codeset_cp932w)) {
 	    kana_force_conv(c1);
 	    enbl = TRUE;
@@ -1798,6 +2007,29 @@ static int hex_conv_table_s[16] = {
   '0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'
 };
 
+void utf32_to_hex(c1,mode)
+skf_ucode c1;
+int	mode;	/* Mode: 0 normal, 1 for java, 2 for hex */
+{
+    int *ctable;
+
+    if (mode == 1) ctable = hex_conv_table;
+    else	   ctable = hex_conv_table_s;
+
+    if (c1 >= 0x10000) {
+	SKFputc(ctable[(c1 >> 20) & 0x0fU]);
+	SKFputc(ctable[(c1 >> 16) & 0x0fU]);
+    } else;
+    if ((c1 >= 0x100) || (mode == 1)) {
+	SKFputc(ctable[(c1 >> 12) & 0x0fU]);
+    } else;
+    if ((c1 >= 0x100) || (mode == 1) || (mode == 2)) {
+	SKFputc(ctable[(c1 >> 8) & 0x0fU]);
+    } else;
+    SKFputc(ctable[(c1 >> 4) & 0x0fU]);
+    SKFputc(ctable[(c1 & 0x0fU)]);
+}
+
 int latin2html(c1)
 skf_ucode c1;
 {
@@ -1809,7 +2041,7 @@ skf_ucode c1;
     if (is_vv_debug)
 		fprintf(stderr," latin2html: %06lx",(unsigned long)c1);
 #endif
-    c2 = (c1 >> 8) & 0xff; c3 = (c1 & 0xff);
+    c2 = (c1 >> 8) & 0xffff; c3 = (c1 & 0xff);
     if (((c1 < A_DEL) && (!use_htmlsanitize ||
 		(uni_latin_html_0[c1 - 0x20] == NULL))) 
 		|| (use_latin2htmld)) {
@@ -1860,6 +2092,14 @@ skf_ucode c1;
 	};
     } else if (c2 == 0x02) {
 	switch (c3) {
+#if 0
+	    case 0x26: sym ="&Adot;"; break;
+	    case 0x27: sym ="&adot;"; break;
+	    case 0x2e: sym ="&Odot;"; break;
+	    case 0x2f: sym ="&odot;"; break;
+	    case 0x32: sym ="&Ymacr;"; break;
+	    case 0x33: sym ="&ymacr;"; break;
+#endif
 	    case 0xbc: sym ="&apos;"; break;
 	    case 0xc6: sym ="&circ;"; break;
 	    case 0xc7: sym ="&caron;"; break;
@@ -1921,17 +2161,7 @@ skf_ucode c1;
 	SKFSTROUT(sym);
     } else {	/* not dec or hex. go hex anyway		  */
 	SKFSTROUT("&#x");
-	if (c1 >= 0x10000) {
-	    SKFputc(hex_conv_table_s[(c1 >> 20) & 0x0fU]);
-	    SKFputc(hex_conv_table_s[(c1 >> 16) & 0x0fU]);
-	    c1 &= 0xffffU;
-	};
-	if (c1 >= 0x100) {
-	    SKFputc(hex_conv_table_s[(c1 >> 12) & 0x0fU]);
-	};
-	SKFputc(hex_conv_table_s[(c1 >> 8) & 0x0fU]);
-	SKFputc(hex_conv_table_s[(c1 >> 4) & 0x0fU]);
-	SKFputc(hex_conv_table_s[(c1 & 0x0fU)]);
+	utf32_to_hex(c1,2);
 	SKFputc(';');
     };
 
