@@ -24,7 +24,7 @@
 **	responsibility for any direct or indirect damages or results
 **	arising by using whole or a part of this software.
 ******************************************************************* */
-/* $Id: skf_convert.h,v 1.10 2015/04/26 15:08:38 seiji Exp seiji $ */
+/* $Id: skf_convert.h,v 1.12 2022/11/12 16:58:38 seiji Exp seiji $ */
 
 #include <setjmp.h>
 
@@ -51,7 +51,12 @@ struct Skf_localestring {
 #ifdef SWIGRUBY
 #define SKFSTRING_IS_VALUE
 #define	HAVE_FAST_LWLSTRLEN
-#if defined(RSTRING_LEN)
+#if defined(SKF_RUBY3) 
+#define get_rstr_len(x)	RSTRING(x)->len
+#define get_rstr_ptr(x) RSTRING(x)->ptr
+#define get_rstr_enc(x) skf_ruby_get_enc(x)
+#define raw_get_rstr_enc(x) STR_ENC_GET(x)
+#elif defined(RSTRING_LEN) 
 #define get_rstr_len(x)	RSTRING_LEN(x)
 #define get_rstr_ptr(x) RSTRING_PTR(x)
 #define get_rstr_enc(x) skf_ruby_get_enc(x)
@@ -78,6 +83,16 @@ struct Skf_localestring {
 #ifndef SKFSTRINGS
 #define SKFSTRINGS	struct Skf_localestring
 #endif
+
+#ifdef USE_PEP393	/* Python 3.3 or later */
+#ifdef Py_UNICODE_WIDE
+#define Py_Default_ocode codeset_utf32le
+#else
+#define Py_Default_ocode codeset_utf16le
+#endif
+#else
+#define Py_Default_ocode DEFAULT_O
+#endif
 #endif	/* SWIGRUBY */
 
 #define		skf_sleep	sleep
