@@ -23,7 +23,7 @@
 **********************************************************************
     ucodoconv.c: output converters for Unicode(TM) variants
 	v1.90	Newly written from unic*de.c parsing
-    $Id: ucodoconv.c,v 1.83 2016/06/14 14:55:50 seiji Exp seiji $
+    $Id: ucodoconv.c,v 1.87 2019/06/28 16:32:03 seiji Exp seiji $
 **/
 
 #include <stdio.h>
@@ -31,6 +31,7 @@
 #include "skf_fileio.h"
 #include "convert.h"
 #include "oconv.h"
+#include "in_code_table.h"
 
 #define  SKFUCODSW	TRUE
 
@@ -41,29 +42,37 @@ static unsigned char utf7_base[64] = {
     'w','x','y','z','0','1','2','3','4','5','6','7','8','9','+','/'
 };
 
+static unsigned char utf7i_base[64] = {
+    'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P',
+    'Q','R','S','T','U','V','W','X','Y','Z','a','b','c','d','e','f',
+    'g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v',
+    'w','x','y','z','0','1','2','3','4','5','6','7','8','9','+',','
+};
+
 /* --- utf7 specific parameters ---------------------------------- */
 static int	utf7_res = 0;	/* utf-7 amari			   */
        int	utf7_res_bit = 0; /* amari bit rength		   */
 
-static void SKFUTF7ENCODE P_((skf_ucode));
+static void SKFUTF7ENCODE P_((skf_ucode,int));
+static void SETSKFUTF7SFT P_((int));
 static void encode_pslenset P_((int *,int *,skf_ucode));
 
 #ifdef ACE_SUPPORT
-int  is_prohibit P_((skf_ucode));
+       int  is_prohibit P_((skf_ucode));
 #endif
 
 /* --- macros ---------------------------------------------------- */
-#define utf8put4(x) SKFputc((int)(ZF4VAL + ((x & ZL4MSK) >> 18))); \
+#define utf8put4(x) {SKFputc((int)(ZF4VAL + ((x & ZL4MSK) >> 18))); \
     SKFputc((int)(ZF2VAL + ((x & ZL43MSK) >> 12)));\
     SKFputc(ZF2VAL + ((x & ZL2MSK) >> 6)); \
-    SKFputc((x & Z1MSK) | ZCVAL);
+    SKFputc((x & Z1MSK) | ZCVAL);}
 
-#define utf8put3(x) SKFputc(ZF3VAL + ((x & ZL3MSK) >> 12));\
+#define utf8put3(x) {SKFputc(ZF3VAL + ((x & ZL3MSK) >> 12));\
     SKFputc(ZF2VAL + ((x & ZL2MSK) >> 6)); \
-    SKFputc((x & Z1MSK) | ZCVAL);
+    SKFputc((x & Z1MSK) | ZCVAL);}
 
-#define utf8put2(x) SKFputc(ZF23VAL + ((x & ZL23MSK) >> 6)); \
-	SKFputc((x & Z1MSK) | ZCVAL);
+#define utf8put2(x) {SKFputc(ZF23VAL + ((x & ZL23MSK) >> 6)); \
+	SKFputc((x & Z1MSK) | ZCVAL);}
 
 #define UCS2PUT2(x,y) {if (out_endian(conv_cap)) \
 	    { SKFputc(x); SKFputc(y); \
@@ -73,6 +82,13 @@ int  is_prohibit P_((skf_ucode));
 	    { SKFputc(x); SKFputc(y); SKFputc(p); SKFputc(q); \
 	    } else { SKFputc(q); SKFputc(p); SKFputc(y); SKFputc(x);}; }
 
+#define utf8cesu8(x) {SKFputc(CESU8VL1);\
+    SKFputc(((x >> 16) & Z3MSK) | CESU8VL2); \
+    SKFputc(((x >> 10) & Z1MSK) | ZCVAL); \
+    SKFputc(CESU8VL1); \
+    SKFputc(((x >> 6) & Z3MSK) | CESU8VL3); \
+    SKFputc((x & Z1MSK) | ZCVAL);}
+
 /* *****************************************************************
 ** Unified Unicode output Converters
 **
@@ -87,18 +103,22 @@ int  is_prohibit P_((skf_ucode));
 ****************************************************************** */
 /* --- unified output 1 ------------------------------------------ */
 /* --- c1 must be 0 <= c1 < 0x10000 ------------------------------ */
-void SKFUNI1OUT(c3)
+static void SKFUNI1OUT(c3)
 skf_ucode	c3;
 {
     int		c1,c2;
 
     if (is_ucs_utf8(conv_cap)) {
 	if (c3 < 0x80) {
-	    SKFputc(c3); 
+	    if ((c3 == 0) && (out_codeset == codeset_jmutf8)) {
+		utf8put2(c3);
+	    } else SKFputc(c3);
+#if 0
 	} else if (c3 < 0x800) {
 	    utf8put2(c3);
 	} else {
 	    utf8put3(c3);
+#endif
 	};
     } else if (is_ucs_utf16(conv_cap)) {
     	if (is_ucs_utf32(conv_cap)) {
@@ -114,25 +134,33 @@ skf_ucode	c3;
     } else if (is_ucs_puny(conv_cap)) {
     	if (!is_prohibit(c3) || is_puny_strdelim(c3)) o_p_encode(c3);
 	else { /* idn prohibit table (rfc3454) is discarded	   */
-	    o_p_encode(A_SP);
+	    o_p_encode(sFLSH);
 	    out_undefined(c3,SKF_OUT_PROHIBIT);
 	};
 #endif
+    } else if (is_ucs_utf7i(conv_cap) && (c3 == 0x26)) {
+    		/* UTF7IMAP 0x26(escape) */
+	SKFputc(c3);SKFputc('-');
     } else if (	/* UTF7: this routine loops due to 0x26	*/
-	   ((c3 >= 0x21) && (c3 <= 0x26))
+        (is_ucs_utf7i(conv_cap) && ((c3 < A_SP) || (c3 >= A_DEL))) ||
+        (is_ucs_utf7(conv_cap) && (
+	   ((c3 < 0x20) && (c3 != A_HT) && (c3 != A_CR) && (c3 != A_LF))
+	|| ((c3 >= 0x21) && (c3 <= 0x26))
 	|| ((c3 >= 0x2a) && (c3 <= 0x2b))
 	|| ((c3 >= 0x3b) && (c3 <= 0x3e))
 	|| ((c3 >= 0x5b) && (c3 <= 0x60))
-	|| (c3 >= 0x7b)) {	/* must be encoded		*/
+	|| (c3 >= 0x7b)))) {	/* must be encoded		*/
 	if (!is_utf7_shift) {
-	    set_utf7_shift; SKFputc('+');
+	    SETSKFUTF7SFT(is_ucs_utf7i(conv_cap));
 	    utf7_res_bit = 0;
 	};
-	SKFUTF7ENCODE(c3);
+	SKFUTF7ENCODE(c3,is_ucs_utf7i(conv_cap));
     } else {
 	if (is_utf7_shift) {
 	    if (utf7_res_bit != 0) {
-		SKFputc(utf7_base[utf7_res]);
+	    	if (is_ucs_utf7(conv_cap)) 
+			SKFputc(utf7_base[utf7_res]);
+		else SKFputc(utf7i_base[utf7_res]);
 	    };
 	    utf7_res_bit = 0;
 	    reset_kanji_shift; SKFputc('-');
@@ -161,7 +189,10 @@ void UNI_ascii_oconv(ch)
     c1 = ch & 0xff;	/* guarding				   */
     c3 = uni_o_ascii[c1];
 #ifdef SKFDEBUG
-    if (is_vv_debug) fprintf(stderr," uni_ascii:%02x",ch);
+    if (is_vv_debug) {
+    	fprintf(stderr," uni_ascii:%02x",ch);
+	debugcharout((int)c3);
+    };
 #endif
     if (o_encode) {
     	out_UNI_encode(ch,c3);
@@ -216,16 +247,16 @@ void UNI_latin_oconv (ch)
 #ifdef ACE_SUPPORT
     } else if (is_ucs_puny(conv_cap)) {
     	if (!is_prohibit(ch)) o_p_encode(ch);
-	else { /* idn prohibit table (rfch454) is discarded	   */
+	else { /* idn prohibit table (rfc3454) is discarded	   */
 	    o_p_encode(A_SP);
 	    out_undefined(ch,SKF_OUT_PROHIBIT);
 	};
 #endif
-    } else if (is_ucs_utf7(conv_cap)) {
+    } else if (is_ucs_utf7m(conv_cap)) {
 	if (!is_utf7_shift) {
-	    set_utf7_shift; SKFputc('+');
+	    SETSKFUTF7SFT(is_ucs_utf7i(conv_cap));
 	};
-	SKFUTF7ENCODE(ch);
+	SKFUTF7ENCODE(ch,(is_ucs_utf7i(conv_cap)));
     };
 }
 
@@ -275,24 +306,23 @@ void UNI_cjkkana_oconv (ch)
 	};
     } else if (is_ucs_utf8(conv_cap)) {
 	utf8put3(ch);
-    } else if (is_ucs_utf7(conv_cap)) {
+    } else if (is_ucs_utf7m(conv_cap)) {
 	if (!is_utf7_shift) {
-	    set_utf7_shift; SKFputc('+');
+	    SETSKFUTF7SFT(is_ucs_utf7i(conv_cap));
 	};
-	SKFUTF7ENCODE(ch);
+	SKFUTF7ENCODE(ch,(is_ucs_utf7i(conv_cap)));
 #ifdef ACE_SUPPORT
     } else if (is_ucs_puny(conv_cap)) {
     	if (!is_prohibit(ch)) o_p_encode(ch);
-	else { /* idn prohibit table (rfc3454) is discarded	   */
+	else {
 	    o_p_encode(A_SP);
 	    out_undefined(ch,SKF_OUT_PROHIBIT);
-	};
+	}; /* idn prohibit table (rfc3454) is discarded	   */
 #endif
     } else;
 }
 
 /* --- CJK plane ------------------------------------------------- */
-/*@-globstate@*/
 void UNI_cjk_oconv(c1)
 skf_ucode c1;
 {
@@ -337,24 +367,23 @@ skf_ucode c1;
 	} else;
 
 	utf8put3(c1);
-    } else if (is_ucs_utf7(conv_cap)) {
+    } else if (is_ucs_utf7m(conv_cap)) {
 	if (!is_utf7_shift) {
-	    set_utf7_shift; SKFputc('+');
+	    SETSKFUTF7SFT(is_ucs_utf7i(conv_cap));
 	};
-	SKFUTF7ENCODE(c1);
+	SKFUTF7ENCODE(c1,(is_ucs_utf7i(conv_cap)));
 #ifdef ACE_SUPPORT
     } else if (is_ucs_puny(conv_cap)) {
     	if (!is_prohibit(c1)) o_p_encode(c1);
-	else { /* idn prohibit table (rfc3454) is discarded	   */
+	else {
 	    o_p_encode(A_SP);
 	    out_undefined(c1,SKF_OUT_PROHIBIT);
-	};
+	}; /* idn prohibit table (rfc3454) is discarded	   */
 #endif
     };
 }
 
 /* --- ozone and non-BMP ----------------------------------------- */
-/*@-globstate@*/
 void UNI_ozone_oconv(ch)
 skf_ucode ch;
 {
@@ -367,6 +396,10 @@ skf_ucode ch;
 #endif
     if (o_encode) out_UNI_encode(ch,ch);
 
+    if ((ch >= VARSEL_SUP_TBL_START) && (ch <= VARSEL_SUP_TBL_END)) {
+    	if (!enable_var_cntl) return;	/* do nothing		   */
+    } else;
+
     if ((out_codeset == codeset_nyukan_utf16) ||
 	(out_codeset == codeset_nyukan_utf8)) {
 	if ((ch >= CJK_B_TBL_START) && (ch <= CJK_B_TBL_END)
@@ -400,8 +433,6 @@ skf_ucode ch;
 		c2 = ((ch >> 8) & 0x3) + 0xdc;
 		c4 = ch & 0xff;
 		UCS2PUT2(c2,c4);
-	    } else if (ch > UNI_HIGH_LIMIT) {
-		out_undefined(ch,SKF_UNSURG);
 	    } else if (limit_ucs2 && (ch >= 0x10000)) {
 		out_undefined(ch,SKF_UCOMPAT);
 	    } else {	/* hangul anf Yi */
@@ -410,37 +441,43 @@ skf_ucode ch;
 		UCS2PUT2(c2,c4);
 	    };
 	};
-    } else if (is_ucs_utf7(conv_cap)) {
+    } else if (is_ucs_utf7m(conv_cap)) {
 	if ((ch <= UNI_HIGH_LIMIT) && (ch >= 0x10000) && !limit_ucs2) {
+	    if (!is_utf7_shift) {
+		SETSKFUTF7SFT(is_ucs_utf7i(conv_cap));
+	    };
 	    /* generate surrogate pair			   */
 	    c3 = ((ch >> 10) - 0x040) & 0x3ff;
 	    c2 = (int) (((c3 >> 8) & 0x3) + 0xd8);
 	    c4 = (int) (c3 & 0xff) + (c2 << 8);
-	    SKFUTF7ENCODE(c4);
+	    SKFUTF7ENCODE(c4,(is_ucs_utf7i(conv_cap)));
 	    c2 = ((ch >> 8) & 0x3) + 0xdc;
 	    c4 = (ch & 0xff) + (c2 << 8);
-	    SKFUTF7ENCODE(c4);
+	    SKFUTF7ENCODE(c4,(is_ucs_utf7i(conv_cap)));
 	} else if (ch > UNI_HIGH_LIMIT) {
 	    out_undefined(ch,SKF_UNSURG);
 	} else if (limit_ucs2 && (ch >= 0x10000)) {
 	    out_undefined(ch,SKF_UCOMPAT);
 	} else {	/* hangul anf Yi */
 	    if (!is_utf7_shift) {
-		set_utf7_shift; SKFputc('+');
+		SETSKFUTF7SFT(is_ucs_utf7i(conv_cap));
 	    };
-	    SKFUTF7ENCODE(ch);
+	    SKFUTF7ENCODE(ch,(is_ucs_utf7i(conv_cap)));
 	};
 #ifdef ACE_SUPPORT
     } else if (is_ucs_puny(conv_cap)) {
     	if (!is_prohibit(ch)) o_p_encode(ch);
-	else { /* idn prohibit table (rfc3454) is discarded	   */
+	else {
 	    o_p_encode(A_SP);
 	    out_undefined(ch,SKF_OUT_PROHIBIT);
-	};
+	}; /* idn prohibit table (rfc3454) is discarded	   */
 #endif
     } else {
 	if ((ch <= UNI_HIGH_LIMIT) && (ch >= 0x10000) && !limit_ucs2) {
-	    utf8put4(ch);
+	    if ((out_codeset == codeset_cesu8) ||
+	        (out_codeset == codeset_jmutf8)) {
+		utf8cesu8(ch);
+	    } else utf8put4(ch);
 	} else if ((ch < 0xd800) && (ch >= 0xa000)) {
 	    utf8put3(ch);
 	} else 
@@ -450,12 +487,17 @@ skf_ucode ch;
 
 
 /* --- private plane --------------------------------------------- */
-/*@-globstate@*/
-void UNI_private_oconv(c1)
+/* - 0xd800 - 0xf8ff --------------------------------------------- */
+void UNI_private_oconv(c1,rc2)
 skf_ucode c1;
+skf_ucode rc2;
 {
     int	c3,c4;
     unsigned long nc;
+#if defined(ARIBINTSUPPORT)
+    skf_ucode *pmap;
+    skf_ucode cnvc;
+#endif
 
     c3 = (c1 >> 8) & 0xff;
     c4 = c1 & 0xff;
@@ -480,20 +522,33 @@ skf_ucode c1;
     if (o_encode) out_UNI_encode(c1,c1);
 
     if (c1 < 0xe000) {
-	lig_x0213_out(c1);
+	lig_x0213_out(c1,rc2);
 	return;
-    };
+#if defined(ARIBINTSUPPORT)
+    } else if ((out_codeset == codeset_aribb24u)
+    	    || (out_codeset == codeset_aribb24z)) {
+	if (arib_bmp0_convert) {
+	    pmap = ovlay_byte_defs[arib_bmp_index].uniltbl;
+	} else if (arib_bmp1_convert) {
+	    pmap = ovlay_byte_defs[arib_bmp1_index].uniltbl;
+	} else pmap = ovlay_byte_defs[arib_bmp2_index].uniltbl;
+	if ((cnvc = pmap[c1 - 0xe000]) != 0) {
+	    post_oconv(cnvc);
+	    return;
+	} else;
+#endif
+    } else;
     if (is_ucs_utf16(conv_cap)) {
     	if (is_ucs_utf32(conv_cap)) {
 	    UCS4PUT4(0,0,c3,c4);
 	} else {
 	    UCS2PUT2(c3,c4);
 	};
-    } else if (is_ucs_utf7(conv_cap)) {
+    } else if (is_ucs_utf7m(conv_cap)) {
 	if (!is_utf7_shift) {
-	    set_utf7_shift; SKFputc('+');
+	    SETSKFUTF7SFT(is_ucs_utf7i(conv_cap));
 	};
-	SKFUTF7ENCODE(c1);
+	SKFUTF7ENCODE(c1,(is_ucs_utf7i(conv_cap)));
 #ifdef ACE_SUPPORT
     } else if (is_ucs_puny(conv_cap)) {
     	if (!is_prohibit(c1) || force_private_idn_o) o_p_encode(c1);
@@ -508,8 +563,7 @@ skf_ucode c1;
 }
 
 /* --- compatibility plane --------------------------------------- */
-/* --- U-F900 to U-FFFD ------------------------------------------ */
-/*@-globstate@*/
+/* --- U-F900 to U-FFFF ------------------------------------------ */
 void UNI_compat_oconv (c1)
     skf_ucode    c1;
 { 
@@ -549,7 +603,7 @@ void UNI_compat_oconv (c1)
     	post_oconv(0xa5);
 	return;
     } else if (c1 >= 0xfffe) {
-	out_undefined(c1,SKF_OUNDEF);
+    	out_undefined(c1,SKF_OUNDEF);
 	return;
     } else if ((use_compat) && 
 	((!sup_cjk_cmp) || (c1 < 0xfa30) || (c1 > 0xfa6a))) {
@@ -562,7 +616,7 @@ void UNI_compat_oconv (c1)
 /* --- 0xffxx ---- */
     } else if (c2 < 0x5f) {
     	c3 = 0; c4 = c2 + 0x20U;
-    } else if ((c2 > 0x61) && (c2 < 0xa0)) {
+    } else if ((c2 > 0x61) && (c2 < 0xa0)) { /* won't be here	   */
 	c3 = 0x30; c4 = uni_t_x201[c2-0x61];
     } else if ((c2 >= 0xa0) && (c2 <= 0xdf)) {
 	if (c2 == 0xa0) { c3 = 0x31; c4 = 0x64; }
@@ -603,7 +657,7 @@ void UNI_compat_oconv (c1)
 	} else {
 	    utf8put3(c5);
 	};
-    } else if (is_ucs_utf7(conv_cap)) {
+    } else if (is_ucs_utf7m(conv_cap)) {
 	c5 = (c3 << 8) + c4;
 	if ((c3 != 0)
 	    || ((c4 >= 0x21) && (c4 <= 0x26))
@@ -612,13 +666,13 @@ void UNI_compat_oconv (c1)
 	    || ((c4 >= 0x5b) && (c4 <= 0x60))
 	    || (c4 >= 0x7b)) {	/* must be encoded		*/
 	    if (!is_utf7_shift) {
-		set_utf7_shift; SKFputc('+');
+		SETSKFUTF7SFT(is_ucs_utf7i(conv_cap));
 	    };
-	    SKFUTF7ENCODE(c5);
+	    SKFUTF7ENCODE(c5,is_ucs_utf7i(conv_cap));
 	} else {
 	    if (is_utf7_shift) {
 		if (utf7_res_bit != 0) {
-		    SKFputc(utf7_base[utf7_res]);
+		    SETSKFUTF7SFT(is_ucs_utf7i(conv_cap));
 		};
 		utf7_res_bit = 0;
 		reset_kanji_shift; SKFputc('-');
@@ -635,6 +689,7 @@ void UNI_compat_oconv (c1)
 #endif
     };
 }
+/*@+globstate@*/
 
 /* *****************************************************************
 ** UCS2 specific Converters
@@ -648,39 +703,54 @@ void ucod_finish_procedure()
 }
 /* *****************************************************************
 ** UTF7 specific Converters
+**  0 < c2 < 0x10000						  **
 ****************************************************************** */
-void SKFUTF7ENCODE(c2)
+void SKFUTF7ENCODE(c2,mod)
 skf_ucode c2;
+int mod;
 {
+    static unsigned char *utf7_be;
 #ifdef SKFDEBUG
-    if (is_vv_debug) fprintf(stderr," code: %x, residue:%x(%d)",
-			c2,utf7_res,utf7_res_bit);
+    if (is_vv_debug) fprintf(stderr," code: %x, residue:%x(%d-%c)",
+			c2,utf7_res,utf7_res_bit,(mod ? 'I':'7'));
 #endif
+    if (mod) utf7_be = utf7i_base;
+    else utf7_be = utf7_base;
+
     if (utf7_res_bit == 0) {
 	utf7_res_bit = 4;
-	SKFputc(utf7_base[(c2 & 0xfc00) >> 10]);
-	SKFputc(utf7_base[(c2 & 0x03f0) >> 4]);
+	SKFputc(utf7_be[(c2 & 0xfc00) >> 10]);
+	SKFputc(utf7_be[(c2 & 0x03f0) >> 4]);
 	utf7_res = (c2 & 0x000f) << 2;
     } else if (utf7_res_bit == 4) {
 	utf7_res_bit = 2;
-	SKFputc(utf7_base[((c2 & 0xc000) >> 14) + utf7_res]);
-	SKFputc(utf7_base[(c2 & 0x3f00) >> 8]);
-	SKFputc(utf7_base[(c2 & 0x00fc) >> 2]);
+	SKFputc(utf7_be[((c2 & 0xc000) >> 14) + utf7_res]);
+	SKFputc(utf7_be[(c2 & 0x3f00) >> 8]);
+	SKFputc(utf7_be[(c2 & 0x00fc) >> 2]);
 	utf7_res = (c2 & 0x0003) << 4;
     } else {	/* utf_res_bit == 2 */
 	utf7_res_bit = 0;
-	SKFputc(utf7_base[((c2 & 0xf000) >> 12) + utf7_res]);
-	SKFputc(utf7_base[(c2 & 0x0fc0) >> 6]);
-	SKFputc(utf7_base[c2 & 0x003f]);
+	SKFputc(utf7_be[((c2 & 0xf000) >> 12) + utf7_res]);
+	SKFputc(utf7_be[(c2 & 0x0fc0) >> 6]);
+	SKFputc(utf7_be[c2 & 0x003f]);
     };
 }
 
+void SETSKFUTF7SFT(mod)
+int mod;
+{
+    set_utf7_shift;
+    if (mod) SKFputc('&');
+    else SKFputc('+');
+}
+
 /* --- end process ----------------------------------------------- */
 void utf7_finish_procedure()
 {
     oconv_flush();
     if (utf7_res_bit != 0) {
-	SKFputc(utf7_base[utf7_res]);
+    	if (is_ucs_utf7(conv_cap)) SKFputc(utf7_base[utf7_res]);
+	else SKFputc(utf7i_base[utf7_res]);
     };
     if (is_kanji_shift) {
 	reset_kanji_shift;
@@ -708,6 +778,7 @@ skf_ucode x;
     SKFputc(hex_conv_table[(x & 0x0fU)]);
 }
 
+/*@+ignorequals@*/
 void utf8_uriout(x)
 skf_ucode x;
 {
@@ -727,6 +798,7 @@ skf_ucode x;
     	utf8_urioutstr(((x & Z1MSK) | ZCVAL));
     } else ;
 }
+/*@-ignorequals@*/
 
 #ifdef ACE_SUPPORT
 /* --------------------------------------------------------------- */
